from pathlib import Path

import packaging.utils

from . import test_projects, utils

meson_project = test_projects.new_meson_project()


def test_meson_python_basic(tmp_path: Path, build_frontend_env: dict[str, str]) -> None:
    """Test that cibuildwheel can build a project with meson-python backend."""
    project_dir = tmp_path / "project"

    meson_project.generate(project_dir)

    # build the wheels
    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env=build_frontend_env,
        single_python=True,
    )

    # check that the expected wheels are produced
    # note that the meson test project doesn't support win32.
    is_windows = utils.get_platform() == "windows"
    expected_wheels = utils.expected_wheels(
        "spam", "0.1.0", single_python=True, single_arch=is_windows
    )
    actual_wheels_normalized = {packaging.utils.parse_wheel_filename(w) for w in actual_wheels}
    expected_wheels_normalized = {packaging.utils.parse_wheel_filename(w) for w in expected_wheels}
    assert actual_wheels_normalized == expected_wheels_normalized



from pathlib import Path

import jinja2
import pytest

from . import utils
from .test_projects import TestProject
from .test_projects.c import SPAM_C_TEMPLATE

subdir_package_project = TestProject()

subdir_package_project.files["src/spam/spam.c"] = jinja2.Template(SPAM_C_TEMPLATE)
subdir_package_project.template_context["spam_c_top_level_add"] = ""
subdir_package_project.template_context["spam_c_function_add"] = ""

subdir_package_project.files["src/spam/setup.py"] = r"""
from setuptools import Extension, setup

setup(
    name="spam",
    ext_modules=[Extension('spam', sources=['spam.c'])],
    version="0.1.0",
)
"""

subdir_package_project.files["src/spam/test/run_tests.py"] = r"""
print('run_tests.py executed!')
"""

subdir_package_project.files["bin/before_build.py"] = r"""
print('before_build.py executed!')
"""


def test(capfd: pytest.CaptureFixture[str], tmp_path: Path) -> None:
    project_dir = tmp_path / "project"
    subdir_package_project.generate(project_dir)

    package_dir = Path("src", "spam")
    # build the wheels
    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        package_dir=package_dir,
        add_env={
            "CIBW_BEFORE_BUILD": "python {project}/bin/before_build.py",
            "CIBW_TEST_COMMAND": "python {package}/test/run_tests.py",
        },
        single_python=True,
    )

    # check that the expected wheels are produced
    expected_wheels = utils.expected_wheels("spam", "0.1.0", single_python=True)
    assert set(actual_wheels) == set(expected_wheels)

    captured = capfd.readouterr()
    assert "before_build.py executed!" in captured.out
    assert "run_tests.py executed!" in captured.out



from packaging.specifiers import SpecifierSet

import cibuildwheel.selector
from cibuildwheel.selector import BuildSelector, EnableGroup


def test_build() -> None:
    build_selector = BuildSelector(
        build_config="cp3*-* *-manylinux*", skip_config="", enable=frozenset([EnableGroup.PyPy])
    )

    assert build_selector("cp36-manylinux_x86_64")
    assert build_selector("cp37-manylinux_x86_64")
    assert build_selector("cp310-manylinux_x86_64")
    assert build_selector("cp311-manylinux_x86_64")
    assert build_selector("cp312-manylinux_x86_64")
    assert build_selector("cp313-manylinux_x86_64")
    assert build_selector("cp314-manylinux_x86_64")
    assert build_selector("pp311-manylinux_x86_64")
    assert build_selector("cp36-manylinux_i686")
    assert build_selector("cp37-manylinux_i686")
    assert build_selector("cp36-macosx_intel")
    assert build_selector("cp37-macosx_intel")
    assert build_selector("cp39-macosx_intel")
    assert build_selector("cp39-macosx_universal2")
    assert build_selector("cp39-macosx_arm64")
    assert not build_selector("pp310-macosx_intel")
    assert not build_selector("pp311-macosx_intel")
    assert build_selector("cp36-win32")
    assert build_selector("cp37-win32")
    assert not build_selector("pp310-win32")
    assert not build_selector("pp311-win32")
    assert build_selector("cp36-win_amd64")
    assert build_selector("cp37-win_amd64")
    assert build_selector("cp310-win_amd64")
    assert build_selector("cp311-win_amd64")
    assert build_selector("cp312-win_amd64")
    assert build_selector("cp313-win_amd64")
    assert build_selector("cp314-win_amd64")
    assert not build_selector("pp310-win_amd64")
    assert not build_selector("pp311-win_amd64")


def test_build_filter_pre() -> None:
    build_selector = BuildSelector(
        build_config="cp3*-* *-manylinux*",
        skip_config="",
        enable=frozenset([EnableGroup.CPythonPrerelease, EnableGroup.PyPy]),
    )

    assert build_selector("cp37-manylinux_x86_64")
    assert build_selector("cp313-manylinux_x86_64")
    assert build_selector("cp37-win_amd64")
    assert build_selector("cp313-win_amd64")
    assert not build_selector("cp313t-manylinux_x86_64")


def test_build_filter_pypy() -> None:
    build_selector = BuildSelector(
        build_config="*",
        skip_config="",
        enable=frozenset([EnableGroup.PyPy]),
    )
    assert build_selector("pp311-manylinux_x86_64")
    assert not build_selector("pp310-manylinux_x86_64")
    assert not build_selector("pp38-manylinux_x86_64")
    assert not build_selector("pp39-manylinux_x86_64")


def test_build_filter_pypy_eol() -> None:
    build_selector = BuildSelector(
        build_config="*",
        skip_config="",
        enable=frozenset([EnableGroup.PyPyEoL]),
    )
    assert not build_selector("pp311-manylinux_x86_64")
    assert build_selector("pp310-manylinux_x86_64")
    assert build_selector("pp38-manylinux_x86_64")
    assert build_selector("pp39-manylinux_x86_64")


def test_build_filter_pypy_all() -> None:
    build_selector = BuildSelector(
        build_config="*",
        skip_config="",
        enable=frozenset([EnableGroup.PyPyEoL, EnableGroup.PyPy]),
    )
    assert build_selector("pp310-manylinux_x86_64")
    assert build_selector("pp311-manylinux_x86_64")
    assert build_selector("pp38-manylinux_x86_64")
    assert build_selector("pp39-manylinux_x86_64")


def test_build_filter_pyodide_prerelease() -> None:
    build_selector = BuildSelector(
        build_config="*",
        skip_config="",
        enable=frozenset([EnableGroup.PyodidePrerelease]),
    )
    assert build_selector("cp312-pyodide_wasm32")
    assert build_selector("cp313-pyodide_wasm32")


def test_build_filter_pyodide() -> None:
    build_selector = BuildSelector(
        build_config="*",
        skip_config="",
        enable=frozenset(),
    )
    assert build_selector("cp312-pyodide_wasm32")
    assert build_selector("cp313-pyodide_wasm32")


def test_skip() -> None:
    build_selector = BuildSelector(
        build_config="*",
        skip_config="pp310-* cp3?-manylinux_i686 cp36-win* *-win32",
        enable=frozenset([EnableGroup.PyPy]),
    )

    assert not build_selector("pp310-manylinux_x86_64")
    assert build_selector("pp311-manylinux_x86_64")
    assert not build_selector("pp37-manylinux_i686")
    assert not build_selector("pp38-manylinux_i686")
    assert build_selector("cp36-manylinux_x86_64")
    assert build_selector("cp37-manylinux_x86_64")
    assert not build_selector("cp36-manylinux_i686")
    assert not build_selector("cp37-manylinux_i686")
    assert not build_selector("pp39-macosx_10_6_intel")
    assert build_selector("pp311-macosx_10_6_intel")
    assert build_selector("cp36-macosx_10_6_intel")
    assert build_selector("cp37-macosx_10_6_intel")
    assert not build_selector("cp36-win32")
    assert not build_selector("cp37-win32")
    assert not build_selector("cp36-win_amd64")
    assert build_selector("cp37-win_amd64")


def test_build_and_skip() -> None:
    build_selector = BuildSelector(
        build_config="cp36-* cp37-macosx* *-manylinux*",
        skip_config="pp37-* cp37-manylinux_i686",
        enable=frozenset([EnableGroup.PyPy]),
    )

    assert not build_selector("pp37-manylinux_x86_64")
    assert build_selector("cp36-manylinux_x86_64")
    assert build_selector("cp37-manylinux_x86_64")
    assert not build_selector("pp37-manylinux_i686")
    assert build_selector("cp36-manylinux_i686")
    assert not build_selector("cp37-manylinux_i686")
    assert not build_selector("pp37-macosx_10_6_intel")
    assert build_selector("cp36-macosx_10_6_intel")
    assert build_selector("cp37-macosx_10_6_intel")
    assert not build_selector("pp37-win32")
    assert build_selector("cp36-win32")
    assert not build_selector("cp37-win32")
    assert not build_selector("pp37-win_amd64")
    assert build_selector("cp36-win_amd64")
    assert not build_selector("cp37-win_amd64")


def test_build_braces() -> None:
    build_selector = BuildSelector(build_config="cp{36,37}*", skip_config="")

    assert build_selector("cp36-manylinux_x86_64")
    assert build_selector("cp37-manylinux_x86_64")
    assert not build_selector("cp38-manylinux_x86_64")
    assert not build_selector("cp39-manylinux_x86_64")


def test_build_limited_python() -> None:
    build_selector = BuildSelector(
        build_config="*",
        skip_config="",
        requires_python=SpecifierSet(">=3.7"),
        enable=frozenset([EnableGroup.PyPy, EnableGroup.PyPyEoL, EnableGroup.GraalPy]),
    )

    assert not build_selector("cp36-manylinux_x86_64")
    assert build_selector("cp37-manylinux_x86_64")
    assert build_selector("cp38-manylinux_x86_64")
    assert not build_selector("cp36-manylinux_i686")
    assert build_selector("cp37-manylinux_i686")
    assert build_selector("cp38-manylinux_i686")
    assert not build_selector("cp36-win32")
    assert build_selector("cp37-win32")
    assert build_selector("cp38-win32")
    assert build_selector("pp37-win_amd64")
    assert build_selector("gp311_242-win_amd64")


def test_build_limited_python_partial() -> None:
    build_selector = BuildSelector(
        build_config="*", skip_config="", requires_python=SpecifierSet(">=3.6, !=3.7.*")
    )

    assert build_selector("cp36-manylinux_x86_64")
    assert not build_selector("cp37-manylinux_x86_64")
    assert build_selector("cp38-manylinux_x86_64")
    assert build_selector("cp39-manylinux_x86_64")


def test_build_limited_python_patch() -> None:
    build_selector = BuildSelector(
        build_config="*", skip_config="", requires_python=SpecifierSet(">=3.6.8")
    )

    assert build_selector("cp36-manylinux_x86_64")
    assert build_selector("cp37-manylinux_x86_64")


def test_build_free_threaded_python() -> None:
    build_selector = BuildSelector(build_config="*", skip_config="", enable=frozenset(EnableGroup))

    assert build_selector("cp313t-manylinux_x86_64")


def test_testing_selector() -> None:
    # This is not a global import to keep pytest from collecting it as a test
    test_selector = cibuildwheel.selector.TestSelector(skip_config="cp36-*")

    assert not test_selector("cp36-win_amd64")
    assert test_selector("cp37-manylinux_x86_64")
    assert test_selector("cp311-manylinux_x86_64")



import platform
from pathlib import Path

import pytest

from . import test_projects, utils

basic_project = test_projects.new_c_project()
basic_project.files["tests/test_suite.py"] = r"""
import platform
print("running tests on " + platform.machine())
"""


ALL_MACOS_WHEELS = {
    *utils.expected_wheels("spam", "0.1.0", machine_arch="x86_64"),
    *utils.expected_wheels("spam", "0.1.0", machine_arch="arm64", include_universal2=True),
}

DEPLOYMENT_TARGET_TOO_LOW_WARNING = "Bumping MACOSX_DEPLOYMENT_TARGET"


def test_cross_compiled_build(tmp_path: Path) -> None:
    if utils.get_platform() != "macos":
        pytest.skip("this test is only relevant to macos")
    if utils.get_xcode_version() < (12, 2):
        pytest.skip("this test only works with Xcode 12.2 or greater")

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={"CIBW_ARCHS": "x86_64, universal2, arm64"},
        single_python=True,
    )
    python_tag = "cp{}{}".format(*utils.SINGLE_PYTHON_VERSION)
    expected_wheels = [w for w in ALL_MACOS_WHEELS if python_tag in w]
    assert set(actual_wheels) == set(expected_wheels)


@pytest.mark.parametrize("build_universal2", [False, True])
@pytest.mark.parametrize(
    "test_config",
    [
        {
            "CIBW_TEST_COMMAND": '''python -c "import platform; print('running tests on ' + platform.machine())"''',
        },
        {
            "CIBW_TEST_COMMAND": "python tests/test_suite.py",
            "CIBW_TEST_SOURCES": "tests",
        },
    ],
)
def test_cross_compiled_test(
    tmp_path: Path,
    capfd: pytest.CaptureFixture[str],
    build_universal2: bool,
    test_config: dict[str, str],
) -> None:
    if utils.get_platform() != "macos":
        pytest.skip("this test is only relevant to macos")
    if utils.get_xcode_version() < (12, 2):
        pytest.skip("this test only works with Xcode 12.2 or greater")

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={
            "CIBW_BUILD": "cp310-*" if build_universal2 else "*p310-*",
            "CIBW_ARCHS": "universal2" if build_universal2 else "x86_64 arm64",
            "CIBW_BUILD_VERBOSITY": "3",
            **test_config,
        },
    )

    captured = capfd.readouterr()

    assert DEPLOYMENT_TARGET_TOO_LOW_WARNING not in captured.err

    platform_machine = platform.machine()
    if platform_machine == "x86_64":
        # ensure that tests were run on only x86_64
        assert "running tests on x86_64" in captured.out
        assert "running tests on arm64" not in captured.out
        if build_universal2:
            assert (
                "While universal2 wheels can be built on x86_64, the arm64 part of the wheel cannot be tested"
                in captured.err
            )
        else:
            assert (
                "While arm64 wheels can be built on x86_64, they cannot be tested" in captured.err
            )
    elif platform_machine == "arm64":
        # ensure that tests were run on both x86_64 and arm64
        assert "running tests on x86_64" in captured.out
        assert "running tests on arm64" in captured.out
        assert (
            "While universal2 wheels can be built on x86_64, the arm64 part of the wheel cannot be tested"
            not in captured.err
        )
        assert (
            "While arm64 wheels can be built on x86_64, they cannot be tested" not in captured.err
        )

    if build_universal2:
        expected_wheels = [w for w in ALL_MACOS_WHEELS if "cp310" in w and "universal2" in w]
    else:
        expected_wheels = [w for w in ALL_MACOS_WHEELS if "p310-" in w and "universal2" not in w]
        if platform_machine == "x86_64":
            expected_wheels = [w for w in expected_wheels if not ("pp310" in w and "arm64" in w)]

    assert set(actual_wheels) == set(expected_wheels)


def test_deployment_target_warning_is_firing(
    tmp_path: Path, capfd: pytest.CaptureFixture[str]
) -> None:
    # force the warning to check that we can detect it if it happens
    if utils.get_platform() != "macos":
        pytest.skip("this test is only relevant to macos")

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    utils.cibuildwheel_run(
        project_dir,
        add_env={
            "CIBW_ARCHS": "x86_64",
            "MACOSX_DEPLOYMENT_TARGET": "10.8",
            "CIBW_BUILD_VERBOSITY": "3",
        },
        single_python=True,
    )

    captured = capfd.readouterr()
    assert DEPLOYMENT_TARGET_TOO_LOW_WARNING in captured.err


@pytest.mark.parametrize("skip_arm64_test", [False, True])
def test_universal2_testing_on_x86_64(
    tmp_path: Path, capfd: pytest.CaptureFixture[str], skip_arm64_test: bool
) -> None:
    if utils.get_platform() != "macos":
        pytest.skip("this test is only relevant to macos")
    if utils.get_xcode_version() < (12, 2):
        pytest.skip("this test only works with Xcode 12.2 or greater")
    if platform.machine() != "x86_64":
        pytest.skip("this test only works on x86_64")

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={
            "CIBW_TEST_COMMAND": '''python -c "import platform; print('running tests on ' + platform.machine())"''',
            "CIBW_ARCHS": "universal2",
            "CIBW_TEST_SKIP": "*_universal2:arm64" if skip_arm64_test else "",
        },
        single_python=True,
    )

    captured = capfd.readouterr()

    if platform.machine() == "x86_64":
        assert "running tests on x86_64" in captured.out
        assert "running tests on arm64" not in captured.out

        warning_message = "While universal2 wheels can be built on x86_64, the arm64 part of the wheel cannot be tested"
        if skip_arm64_test:
            assert warning_message not in captured.err
        else:
            assert warning_message in captured.err

    python_tag = "cp{}{}".format(*utils.SINGLE_PYTHON_VERSION)
    expected_wheels = [w for w in ALL_MACOS_WHEELS if python_tag in w and "universal2" in w]

    assert set(actual_wheels) == set(expected_wheels)


def test_universal2_testing_on_arm64(
    build_frontend_env: dict[str, str], tmp_path: Path, capfd: pytest.CaptureFixture[str]
) -> None:
    # cibuildwheel should test the universal2 wheel on both x86_64 and arm64, when run on arm64
    if utils.get_platform() != "macos":
        pytest.skip("this test is only relevant to macos")
    if platform.machine() != "arm64":
        pytest.skip("this test only works on arm64")

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={
            "CIBW_ARCHS": "universal2",
            # check that a native dependency is correctly installed, once per each testing arch
            "CIBW_TEST_REQUIRES": "--only-binary :all: pillow>=10.3",  # pillow>=10.3 provides wheels for macOS 10.10, not 10.9
            "CIBW_TEST_COMMAND": '''python -c "import PIL, platform; print(f'running tests on {platform.machine()} with pillow {PIL.__version__}')"''',
            **build_frontend_env,
        },
        single_python=True,
    )

    captured = capfd.readouterr()
    assert "running tests on arm64 with pillow" in captured.out
    assert "running tests on x86_64 with pillow" in captured.out

    python_tag = "cp{}{}".format(*utils.SINGLE_PYTHON_VERSION)
    expected_wheels = [w for w in ALL_MACOS_WHEELS if python_tag in w and "universal2" in w]
    assert set(actual_wheels) == set(expected_wheels)


def test_cp38_arm64_testing(
    tmp_path: Path, capfd: pytest.CaptureFixture[str], request: pytest.FixtureRequest
) -> None:
    if utils.get_platform() != "macos":
        pytest.skip("this test is only relevant to macos")
    if utils.get_xcode_version() < (12, 2):
        pytest.skip("this test only works with Xcode 12.2 or greater")
    if platform.machine() != "arm64":
        pytest.skip("this test only works on arm64")
    if request.config.getoption("--run-cp38-universal2"):
        pytest.skip("--run-cp38-universal2 option skips this test")

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={
            "CIBW_BUILD": "cp38-*",
            "CIBW_TEST_COMMAND": '''python -c "import platform; print('running tests on ' + platform.machine())"''',
            "CIBW_ARCHS": "x86_64,universal2,arm64",
        },
    )

    captured = capfd.readouterr()

    assert "running tests on x86_64" in captured.out
    assert "running tests on arm64" not in captured.out

    warning_message = "While cibuildwheel can build CPython 3.8 universal2/arm64 wheels, we cannot test the arm64 part of them"
    assert warning_message in captured.err

    expected_wheels = [w for w in ALL_MACOS_WHEELS if "cp38" in w]

    assert set(actual_wheels) == set(expected_wheels)


def test_cp38_arm64_testing_universal2_installer(
    tmp_path: Path, capfd: pytest.CaptureFixture[str], request: pytest.FixtureRequest
) -> None:
    if not request.config.getoption("--run-cp38-universal2"):
        pytest.skip("needs --run-cp38-universal2 option to run")

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={
            "CIBW_BUILD": "cp38-*",
            "CIBW_TEST_COMMAND": '''python -c "import platform; print('running tests on ' + platform.machine())"''',
            "CIBW_ARCHS": "x86_64,universal2,arm64",
            "MACOSX_DEPLOYMENT_TARGET": "11.0",
        },
    )

    captured = capfd.readouterr()

    assert "running tests on x86_64" in captured.out
    assert "running tests on arm64" in captured.out

    warning_message = "While cibuildwheel can build CPython 3.8 universal2/arm64 wheels, we cannot test the arm64 part of them"
    assert warning_message not in captured.err

    expected_wheels = [w.replace("10_9", "11_0") for w in ALL_MACOS_WHEELS if "cp38" in w]

    assert set(actual_wheels) == set(expected_wheels)



import os
import subprocess
import sys
import textwrap
from collections.abc import Mapping
from pathlib import Path
from tempfile import TemporaryDirectory

import pytest

from test.test_projects.base import TestProject

from . import test_projects, utils

# utilities


def make_sdist(project: TestProject, working_dir: Path) -> Path:
    project_dir = working_dir / "project"
    project_dir.mkdir(parents=True, exist_ok=True)
    project.generate(project_dir)

    sdist_dir = working_dir / "sdist"
    subprocess.run(
        [sys.executable, "-m", "build", "--sdist", "--outdir", str(sdist_dir), str(project_dir)],
        check=True,
    )

    return next(sdist_dir.glob("*.tar.gz"))


def cibuildwheel_from_sdist_run(
    sdist_path: Path | str,
    add_env: Mapping[str, str] | None = None,
    config_file: str | None = None,
) -> list[str]:
    env = os.environ.copy()

    if add_env:
        env.update(add_env)

    env["CIBW_BUILD"] = "cp{}{}-*".format(*utils.SINGLE_PYTHON_VERSION)

    with TemporaryDirectory() as tmp_output_dir:
        subprocess.run(
            [
                sys.executable,
                "-m",
                "cibuildwheel",
                *(["--config-file", config_file] if config_file else []),
                "--output-dir",
                str(tmp_output_dir),
                str(sdist_path),
            ],
            env=env,
            check=True,
        )
        return [p.name for p in Path(tmp_output_dir).iterdir()]


# tests


def test_simple(tmp_path: Path) -> None:
    basic_project = test_projects.new_c_project()

    # make an sdist of the project
    sdist_dir = tmp_path / "sdist"
    sdist_dir.mkdir()
    sdist_path = make_sdist(basic_project, sdist_dir)

    setup_py_assertion_snippet = textwrap.dedent(
        """
        import os

        assert os.path.exists('setup.py')
        assert os.path.exists('{package}/setup.py')
        """,
    )
    setup_py_assertion_cmd = f'python -c "{setup_py_assertion_snippet!s}"'

    # build the wheels from sdist
    actual_wheels = cibuildwheel_from_sdist_run(
        sdist_path, add_env={"CIBW_BEFORE_BUILD": setup_py_assertion_cmd}
    )

    # check that the expected wheels are produced
    expected_wheels = utils.expected_wheels("spam", "0.1.0", single_python=True)
    assert set(actual_wheels) == set(expected_wheels)


def test_external_config_file_argument(tmp_path: Path, capfd: pytest.CaptureFixture[str]) -> None:
    basic_project = test_projects.new_c_project()

    # make an sdist of the project
    sdist_dir = tmp_path / "sdist"
    sdist_dir.mkdir()
    sdist_path = make_sdist(basic_project, sdist_dir)

    # add a config file
    config_file = tmp_path / "config.toml"
    config_file.write_text(
        textwrap.dedent(
            """
            [tool.cibuildwheel]
            before-all = 'echo "test log statement from before-all"'
            """
        )
    )

    # build the wheels from sdist
    actual_wheels = cibuildwheel_from_sdist_run(sdist_path, config_file=str(config_file))

    # check that the expected wheels are produced
    expected_wheels = utils.expected_wheels("spam", "0.1.0", single_python=True)
    assert set(actual_wheels) == set(expected_wheels)

    # check that before-all was run
    captured = capfd.readouterr()
    assert "test log statement from before-all" in captured.out


def test_config_in_pyproject_toml(tmp_path: Path, capfd: pytest.CaptureFixture[str]) -> None:
    # make a project with a pyproject.toml
    project = test_projects.new_c_project()
    project.files["pyproject.toml"] = textwrap.dedent(
        """
        [tool.cibuildwheel]
        before-build = 'echo "test log statement from before-build 8419"'
        """
    )

    # make an sdist of the project
    sdist_dir = tmp_path / "sdist"
    sdist_dir.mkdir()
    sdist_path = make_sdist(project, sdist_dir)

    # build the wheels from sdist
    actual_wheels = cibuildwheel_from_sdist_run(sdist_path)

    # check that the expected wheels are produced
    expected_wheels = utils.expected_wheels("spam", "0.1.0", single_python=True)
    assert set(actual_wheels) == set(expected_wheels)

    # check that before-build was run
    captured = capfd.readouterr()
    assert "test log statement from before-build 8419" in captured.out


def test_internal_config_file_argument(tmp_path: Path, capfd: pytest.CaptureFixture[str]) -> None:
    # make a project with a config file inside
    project = test_projects.new_c_project(
        setup_cfg_add="include_package_data = True",
    )
    project.files["wheel_build_config.toml"] = textwrap.dedent(
        """
        [tool.cibuildwheel]
        before-all = 'echo "test log statement from before-all 1829"'
        """
    )
    project.files["MANIFEST.in"] = textwrap.dedent(
        """
        include wheel_build_config.toml
        """
    )

    # make an sdist of the project
    sdist_dir = tmp_path / "sdist"
    sdist_dir.mkdir()
    sdist_path = make_sdist(project, sdist_dir)

    # build the wheels from sdist, referencing the config file inside
    actual_wheels = cibuildwheel_from_sdist_run(
        sdist_path, config_file="{package}/wheel_build_config.toml"
    )

    # check that the expected wheels are produced
    expected_wheels = utils.expected_wheels("spam", "0.1.0", single_python=True)
    assert set(actual_wheels) == set(expected_wheels)

    # check that before-all was run
    captured = capfd.readouterr()
    assert "test log statement from before-all 1829" in captured.out



import textwrap
from pathlib import Path, PurePath
from unittest.mock import Mock, call

import pytest

from cibuildwheel import errors
from cibuildwheel.ci import fix_ansi_codes_for_github_actions
from cibuildwheel.util.file import copy_test_sources
from cibuildwheel.util.helpers import (
    FlexibleVersion,
    format_safe,
    parse_key_value_string,
    prepare_command,
    unwrap,
    unwrap_preserving_paragraphs,
)
from cibuildwheel.util.packaging import find_compatible_wheel


def test_format_safe() -> None:
    assert format_safe("{wheel}", wheel="filename.whl") == "filename.whl"
    assert format_safe("command #{wheel}", wheel="filename.whl") == "command {wheel}"
    assert format_safe("{command #{wheel}}", wheel="filename.whl") == "{command {wheel}}"

    # check unmatched brackets
    assert format_safe("{command {wheel}", wheel="filename.whl") == "{command filename.whl"

    # check positional-style arguments i.e. {}
    assert (
        format_safe("find . -name  * -exec ls -a {} \\;", project="/project")
        == "find . -name  * -exec ls -a {} \\;"
    )

    assert format_safe("{param} {param}", param="1") == "1 1"
    assert format_safe("# {param} {param}", param="1") == "# 1 1"
    assert format_safe("#{not_a_param} {param}", param="1") == "#{not_a_param} 1"


def test_prepare_command() -> None:
    assert prepare_command("python -m {project}", project="project") == "python -m project"
    assert prepare_command("python -m {something}", project="project") == "python -m {something}"
    assert (
        prepare_command("python -m {something.abc}", project="project")
        == "python -m {something.abc}"
    )

    assert (
        prepare_command("python -m {something.abc[4]:3f}", project="project")
        == "python -m {something.abc[4]:3f}"
    )

    # test backslashes in the replacement
    assert (
        prepare_command(
            "command {wheel} \\Users\\Temp\\output_dir", wheel="\\Temporary Files\\cibw"
        )
        == "command \\Temporary Files\\cibw \\Users\\Temp\\output_dir"
    )

    # test some unusual syntax that used to trip up the str.format approach
    assert (
        prepare_command("{a}{a,b}{b:.2e}{c}{d%s}{e:3}{f[0]}", a="42", b="3.14159")
        == "42{a,b}{b:.2e}{c}{d%s}{e:3}{f[0]}"
    )


@pytest.mark.parametrize(
    ("wheel", "identifier"),
    [
        ("foo-0.1-cp38-abi3-win_amd64.whl", "cp310-win_amd64"),
        ("foo-0.1-cp38-abi3-macosx_11_0_x86_64.whl", "cp310-macosx_x86_64"),
        ("foo-0.1-cp38-abi3-manylinux2014_x86_64.whl", "cp310-manylinux_x86_64"),
        ("foo-0.1-cp38-abi3-musllinux_1_1_x86_64.whl", "cp310-musllinux_x86_64"),
        ("foo-0.1-py2.py3-none-win_amd64.whl", "cp310-win_amd64"),
        ("foo-0.1-py2.py3-none-win_amd64.whl", "pp310-win_amd64"),
        ("foo-0.1-py3-none-win_amd64.whl", "cp310-win_amd64"),
        ("foo-0.1-py38-none-win_amd64.whl", "cp310-win_amd64"),
        ("foo-0.1-py38-none-win_amd64.whl", "pp310-win_amd64"),
    ],
)
def test_find_compatible_wheel_found(wheel: str, identifier: str) -> None:
    wheel_ = PurePath(wheel)
    found = find_compatible_wheel([wheel_], identifier)
    assert found is wheel_


@pytest.mark.parametrize(
    ("wheel", "identifier"),
    [
        ("foo-0.1-cp38-abi3-win_amd64.whl", "cp310-win32"),
        ("foo-0.1-cp38-abi3-win_amd64.whl", "cp37-win_amd64"),
        ("foo-0.1-cp38-abi3-macosx_11_0_x86_64.whl", "cp310-macosx_universal2"),
        ("foo-0.1-cp38-abi3-manylinux2014_x86_64.whl", "cp310-musllinux_x86_64"),
        ("foo-0.1-cp38-abi3-musllinux_1_1_x86_64.whl", "cp310-manylinux_x86_64"),
        ("foo-0.1-py2-none-win_amd64.whl", "cp310-win_amd64"),
        ("foo-0.1-py38-none-win_amd64.whl", "cp37-win_amd64"),
        ("foo-0.1-py38-none-win_amd64.whl", "pp37-win_amd64"),
        ("foo-0.1-cp38-cp38-win_amd64.whl", "cp310-win_amd64"),
    ],
)
def test_find_compatible_wheel_not_found(wheel: str, identifier: str) -> None:
    assert find_compatible_wheel([PurePath(wheel)], identifier) is None


def test_fix_ansi_codes_for_github_actions() -> None:
    input = textwrap.dedent(
        """
        This line is normal
        \033[1mThis line is bold
        This line is also bold
        \033[31m this line is red and bold
        This line is red and bold, too\033[0m
        This line is normal again
        """
    )

    expected = textwrap.dedent(
        """
        This line is normal
        \033[1mThis line is bold
        \033[1mThis line is also bold
        \033[1m\033[31m this line is red and bold
        \033[1m\033[31mThis line is red and bold, too\033[0m
        This line is normal again
        """
    )

    output = fix_ansi_codes_for_github_actions(input)

    assert output == expected


def test_parse_key_value_string() -> None:
    assert parse_key_value_string("bar", positional_arg_names=["foo"]) == {"foo": ["bar"]}
    assert parse_key_value_string("foo:bar", kw_arg_names=["foo"]) == {"foo": ["bar"]}
    with pytest.raises(ValueError, match="Too many positional arguments"):
        parse_key_value_string("bar")
    with pytest.raises(ValueError, match="Unknown field name"):
        parse_key_value_string("foo:bar")
    assert parse_key_value_string("foo:bar", kw_arg_names=["foo"]) == {"foo": ["bar"]}
    assert parse_key_value_string("foo:bar", positional_arg_names=["foo"]) == {"foo": ["bar"]}
    assert parse_key_value_string("foo: bar", kw_arg_names=["foo"]) == {"foo": ["bar"]}
    assert parse_key_value_string("foo: bar", kw_arg_names=["foo"]) == {"foo": ["bar"]}
    assert parse_key_value_string("foo: bar; baz: qux", kw_arg_names=["foo", "baz"]) == {
        "foo": ["bar"],
        "baz": ["qux"],
    }

    # some common options
    assert parse_key_value_string(
        "docker; create_args: --some-option --another-option=foo",
        positional_arg_names=["name"],
        kw_arg_names=["create_args"],
    ) == {
        "name": ["docker"],
        "create_args": ["--some-option", "--another-option=foo"],
    }
    # semicolon in value
    assert parse_key_value_string(
        "docker; create_args: --some-option='this; that'",
        positional_arg_names=["name"],
        kw_arg_names=["create_args"],
    ) == {
        "name": ["docker"],
        "create_args": ["--some-option=this; that"],
    }
    # colon in value
    assert parse_key_value_string(
        "docker; create_args: --mount a:b",
        positional_arg_names=["name"],
        kw_arg_names=["create_args"],
    ) == {
        "name": ["docker"],
        "create_args": ["--mount", "a:b"],
    }
    assert parse_key_value_string(
        "docker;create_args:--mount a:b",
        positional_arg_names=["name"],
        kw_arg_names=["create_args"],
    ) == {
        "name": ["docker"],
        "create_args": ["--mount", "a:b"],
    }
    # quoted value with spaces
    assert parse_key_value_string(
        "docker;create_args:'some string with spaces'",
        positional_arg_names=["name"],
        kw_arg_names=["create_args"],
    ) == {
        "name": ["docker"],
        "create_args": ["some string with spaces"],
    }

    # colon in positional value
    assert parse_key_value_string(
        "docker; --mount a:b",
        positional_arg_names=["name", "create_args"],
    ) == {
        "name": ["docker"],
        "create_args": ["--mount", "a:b"],
    }

    # empty option gives empty array
    assert parse_key_value_string(
        "docker;create_args:",
        positional_arg_names=["name"],
        kw_arg_names=["create_args"],
    ) == {
        "name": ["docker"],
        "create_args": [],
    }


def test_flexible_version_comparisons() -> None:
    assert FlexibleVersion("2.0") == FlexibleVersion("2")
    assert FlexibleVersion("2.0") < FlexibleVersion("2.1")
    assert FlexibleVersion("2.1") > FlexibleVersion("2")
    assert FlexibleVersion("1.9.9") < FlexibleVersion("2.0")
    assert FlexibleVersion("1.10") > FlexibleVersion("1.9.9")
    assert FlexibleVersion("3.0.1") > FlexibleVersion("3.0")
    assert FlexibleVersion("3.0") < FlexibleVersion("3.0.1")
    # Suffix should not affect comparisons
    assert FlexibleVersion("1.0.1-rhel") > FlexibleVersion("1.0")
    assert FlexibleVersion("1.0.1-rhel") < FlexibleVersion("1.1")
    assert FlexibleVersion("1.0.1") == FlexibleVersion("v1.0.1")


@pytest.fixture
def sample_project(tmp_path: Path) -> Path:
    """Create a directory structure that contains a range of files."""
    project_path = tmp_path / "project"

    (project_path / "src/deep").mkdir(parents=True)
    (project_path / "tests/deep").mkdir(parents=True)
    (project_path / "other").mkdir(parents=True)

    (project_path / "pyproject.toml").write_text("A pyproject.toml file")
    (project_path / "test.cfg").write_text("A test config file")

    (project_path / "src/__init__.py").write_text("source init")
    (project_path / "src/module.py").write_text("source module")
    (project_path / "src/deep/__init__.py").write_text("deep source init")

    (project_path / "tests/test_module.py").write_text("test module")
    (project_path / "tests/deep/test_module.py").write_text("deep test module")
    (project_path / "tests/deep/__init__.py").write_text("deep test init")

    (project_path / "other/module.py").write_text("other module")

    return project_path


@pytest.mark.parametrize(
    ("test_sources", "expected", "not_expected"),
    [
        # Empty test_sources copies nothing.
        pytest.param(
            [],
            [],
            [
                "pyproject.toml",
                "test.cfg",
                "other/module.py",
                "src/__init__.py",
                "src/module.py",
                "src/deep/__init__.py",
                "tests/test_module.py",
                "tests/deep/__init__.py",
                "tests/deep/test_module.py",
            ],
            id="empty",
        ),
        # Single standalone files
        pytest.param(
            ["pyproject.toml", "tests/deep/test_module.py"],
            ["pyproject.toml", "tests/deep/test_module.py"],
            [
                "test.cfg",
                "other/module.py",
                "src/__init__.py",
                "src/module.py",
                "src/deep/__init__.py",
                "tests/test_module.py",
                "tests/deep/__init__.py",
            ],
            id="single-file",
        ),
        # A full Directory
        pytest.param(
            ["tests"],
            [
                "tests/test_module.py",
                "tests/deep/__init__.py",
                "tests/deep/test_module.py",
            ],
            [
                "pyproject.toml",
                "test.cfg",
                "other/module.py",
                "src/__init__.py",
                "src/module.py",
                "src/deep/__init__.py",
            ],
            id="top-level-directory",
        ),
        # A partial deep directory
        pytest.param(
            ["tests/deep"],
            [
                "tests/deep/__init__.py",
                "tests/deep/test_module.py",
            ],
            [
                "pyproject.toml",
                "test.cfg",
                "other/module.py",
                "src/__init__.py",
                "src/module.py",
                "src/deep/__init__.py",
                "tests/test_module.py",
            ],
            id="partial-directory",
        ),
    ],
)
def test_copy_test_sources(
    tmp_path: Path,
    sample_project: Path,
    test_sources: list[str],
    expected: list[str],
    not_expected: list[str],
) -> None:
    """Test sources can be copied into the test directory."""
    target = tmp_path / "somewhere/test_cwd"
    copy_test_sources(test_sources, sample_project, target)

    for path in expected:
        assert (tmp_path / "somewhere/test_cwd" / path).is_file()

    for path in not_expected:
        assert not (tmp_path / "somewhere/test_cwd" / path).exists()


def test_copy_test_sources_missing_file(tmp_path: Path, sample_project: Path) -> None:
    """If test_sources references a folder that doesn't exist, an error is raised."""

    with pytest.raises(
        errors.FatalError,
        match=r"Test source tests/does_not_exist.py does not exist.",
    ):
        copy_test_sources(
            ["pyproject.toml", "tests/does_not_exist.py"],
            sample_project,
            tmp_path / "somewhere/test_cwd",
        )


def test_copy_test_sources_alternate_copy_into(sample_project: Path) -> None:
    """If an alternate copy_into method is provided, it is used."""

    target = PurePath("/container/test_cwd")
    copy_into = Mock()

    copy_test_sources(["pyproject.toml", "tests"], sample_project, target, copy_into=copy_into)

    copy_into.assert_has_calls(
        [
            call(sample_project / "pyproject.toml", target / "pyproject.toml"),
            call(sample_project / "tests", target / "tests"),
        ],
        any_order=True,
    )


def test_unwrap() -> None:
    assert (
        unwrap("""
            This is a
            multiline
            string
        """)
        == "This is a multiline string"
    )


def test_unwrap_preserving_paragraphs() -> None:
    assert (
        unwrap("""
            This is a
            multiline
            string
        """)
        == "This is a multiline string"
    )
    assert (
        unwrap_preserving_paragraphs("""
            paragraph one

            paragraph two
        """)
        == "paragraph one\n\nparagraph two"
    )



import sys
import textwrap
from pathlib import Path

import pytest
from packaging.specifiers import SpecifierSet

from cibuildwheel.__main__ import main

from .conftest import ArgsInterceptor


@pytest.fixture(autouse=True)
def fake_package_dir(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> Path:
    """
    Set up a fake project
    """

    local_path = tmp_path / "tmp_project"
    local_path.mkdir()

    local_path.joinpath("setup.py").touch()

    monkeypatch.setattr(sys, "argv", ["cibuildwheel", str(local_path)])

    return local_path


@pytest.mark.usefixtures("platform")
def test_no_override(intercepted_build_args: ArgsInterceptor) -> None:
    main()

    options = intercepted_build_args.args[0]
    intercepted_build_selector = options.globals.build_selector

    assert intercepted_build_selector("cp39-win32")
    assert intercepted_build_selector("cp36-win32")

    assert intercepted_build_selector.requires_python is None


@pytest.mark.usefixtures("platform")
def test_override_env(
    monkeypatch: pytest.MonkeyPatch, intercepted_build_args: ArgsInterceptor
) -> None:
    monkeypatch.setenv("CIBW_PROJECT_REQUIRES_PYTHON", ">=3.8")

    main()

    options = intercepted_build_args.args[0]
    intercepted_build_selector = options.globals.build_selector

    assert intercepted_build_selector.requires_python == SpecifierSet(">=3.8")

    assert intercepted_build_selector("cp39-win32")
    assert not intercepted_build_selector("cp36-win32")


@pytest.mark.usefixtures("platform")
def test_override_setup_cfg(
    intercepted_build_args: ArgsInterceptor, fake_package_dir: Path
) -> None:
    fake_package_dir.joinpath("setup.cfg").write_text(
        textwrap.dedent(
            """
            [options]
            python_requires = >=3.8
            """
        )
    )

    main()

    options = intercepted_build_args.args[0]
    intercepted_build_selector = options.globals.build_selector

    assert intercepted_build_selector.requires_python == SpecifierSet(">=3.8")

    assert intercepted_build_selector("cp39-win32")
    assert not intercepted_build_selector("cp36-win32")


@pytest.mark.usefixtures("platform")
def test_override_pyproject_toml(
    intercepted_build_args: ArgsInterceptor, fake_package_dir: Path
) -> None:
    fake_package_dir.joinpath("pyproject.toml").write_text(
        textwrap.dedent(
            """
            [project]
            requires-python = ">=3.8"
            """
        )
    )

    main()

    options = intercepted_build_args.args[0]
    intercepted_build_selector = options.globals.build_selector

    assert intercepted_build_selector.requires_python == SpecifierSet(">=3.8")

    assert intercepted_build_selector("cp39-win32")
    assert not intercepted_build_selector("cp36-win32")


@pytest.mark.usefixtures("platform")
def test_override_setup_py_simple(
    intercepted_build_args: ArgsInterceptor, fake_package_dir: Path
) -> None:
    fake_package_dir.joinpath("setup.py").write_text(
        textwrap.dedent(
            """
            from setuptools import setup

            setup(
                name = "other",
                python_requires = ">=3.7",
            )
            """
        )
    )

    main()

    options = intercepted_build_args.args[0]
    intercepted_build_selector = options.globals.build_selector

    assert intercepted_build_selector.requires_python == SpecifierSet(">=3.7")

    assert intercepted_build_selector("cp39-win32")
    assert not intercepted_build_selector("cp36-win32")



import sys

import pytest

from cibuildwheel.__main__ import main
from cibuildwheel.architecture import Architecture
from cibuildwheel.selector import EnableGroup
from cibuildwheel.typing import PlatformName

from ..conftest import MOCK_PACKAGE_DIR
from .conftest import ArgsInterceptor


@pytest.mark.parametrize("option_value", [None, "auto", ""])
def test_platform_unset_or_auto(
    monkeypatch: pytest.MonkeyPatch,
    intercepted_build_args: ArgsInterceptor,
    option_value: str | None,
) -> None:
    if option_value is None:
        monkeypatch.delenv("CIBW_PLATFORM", raising=False)
    else:
        monkeypatch.setenv("CIBW_PLATFORM", option_value)

    main()

    options = intercepted_build_args.args[0]

    # check that the platform was auto detected to build for the current system
    if sys.platform.startswith("linux"):
        assert options.platform == "linux"
    elif sys.platform == "darwin":
        assert options.platform == "macos"
    elif sys.platform == "win32":
        assert options.platform == "windows"
    else:
        pytest.fail(f"Unknown platform: {sys.platform}")


def test_unknown_platform_on_ci(
    monkeypatch: pytest.MonkeyPatch, capsys: pytest.CaptureFixture[str]
) -> None:
    monkeypatch.setenv("CI", "true")
    monkeypatch.setattr(sys, "platform", "nonexistent")
    monkeypatch.delenv("CIBW_PLATFORM", raising=False)

    with pytest.raises(SystemExit) as exit:
        main()
    assert exit.value.code == 2
    _, err = capsys.readouterr()

    assert 'Unable to detect platform from "sys.platform"' in err


def test_unknown_platform(
    monkeypatch: pytest.MonkeyPatch, capsys: pytest.CaptureFixture[str]
) -> None:
    monkeypatch.setenv("CIBW_PLATFORM", "nonexistent")

    with pytest.raises(SystemExit) as exit:
        main()
    _, err = capsys.readouterr()

    assert exit.value.code == 2
    assert "Unsupported platform: nonexistent" in err


def test_platform_argument(
    platform: str, intercepted_build_args: ArgsInterceptor, monkeypatch: pytest.MonkeyPatch
) -> None:
    monkeypatch.setenv("CIBW_PLATFORM", "nonexistent")
    monkeypatch.setattr(sys, "argv", [*sys.argv, "--platform", platform])

    main()

    options = intercepted_build_args.args[0]

    assert options.globals.package_dir == MOCK_PACKAGE_DIR.resolve()


@pytest.mark.usefixtures("platform")
def test_platform_environment(intercepted_build_args: ArgsInterceptor) -> None:
    main()
    options = intercepted_build_args.args[0]

    assert options.globals.package_dir == MOCK_PACKAGE_DIR.resolve()


def test_archs_default(platform: str, intercepted_build_args: ArgsInterceptor) -> None:
    main()
    options = intercepted_build_args.args[0]

    if platform == "linux":
        assert options.globals.architectures == {Architecture.x86_64}
    elif platform == "windows":
        assert options.globals.architectures == {Architecture.AMD64, Architecture.x86}
    else:
        assert options.globals.architectures == {Architecture.x86_64}


@pytest.mark.parametrize("use_env_var", [False, True])
def test_archs_argument(
    platform: str,
    intercepted_build_args: ArgsInterceptor,
    monkeypatch: pytest.MonkeyPatch,
    use_env_var: bool,
) -> None:
    if use_env_var:
        monkeypatch.setenv("CIBW_ARCHS", "ppc64le")
    else:
        monkeypatch.setenv("CIBW_ARCHS", "unused")
        monkeypatch.setattr(sys, "argv", [*sys.argv, "--archs", "ppc64le"])

    if platform in {"macos", "windows"}:
        with pytest.raises(SystemExit) as exit:
            main()
        assert exit.value.args == (4,)

    else:
        main()
        options = intercepted_build_args.args[0]
        assert options.globals.architectures == {Architecture.ppc64le}


def test_archs_platform_specific(
    platform: str, intercepted_build_args: ArgsInterceptor, monkeypatch: pytest.MonkeyPatch
) -> None:
    monkeypatch.setenv("CIBW_ARCHS", "unused")
    monkeypatch.setenv("CIBW_ARCHS_LINUX", "ppc64le")
    monkeypatch.setenv("CIBW_ARCHS_WINDOWS", "x86")
    monkeypatch.setenv("CIBW_ARCHS_MACOS", "x86_64")

    main()
    options = intercepted_build_args.args[0]

    if platform == "linux":
        assert options.globals.architectures == {Architecture.ppc64le}
    elif platform == "windows":
        assert options.globals.architectures == {Architecture.x86}
    elif platform == "macos":
        assert options.globals.architectures == {Architecture.x86_64}


def test_archs_platform_native(
    platform: str, intercepted_build_args: ArgsInterceptor, monkeypatch: pytest.MonkeyPatch
) -> None:
    monkeypatch.setenv("CIBW_ARCHS", "native")

    main()
    options = intercepted_build_args.args[0]

    if platform in {"linux", "macos"}:
        assert options.globals.architectures == {Architecture.x86_64}
    elif platform == "windows":
        assert options.globals.architectures == {Architecture.AMD64}


def test_archs_platform_auto64(
    platform: str, intercepted_build_args: ArgsInterceptor, monkeypatch: pytest.MonkeyPatch
) -> None:
    monkeypatch.setenv("CIBW_ARCHS", "auto64")

    main()
    options = intercepted_build_args.args[0]

    if platform in {"linux", "macos"}:
        assert options.globals.architectures == {Architecture.x86_64}
    elif platform == "windows":
        assert options.globals.architectures == {Architecture.AMD64}


def test_archs_platform_auto32(
    platform: str, intercepted_build_args: ArgsInterceptor, monkeypatch: pytest.MonkeyPatch
) -> None:
    monkeypatch.setenv("CIBW_ARCHS", "auto32")

    if platform == "macos":
        with pytest.raises(SystemExit) as exit:
            main()
        assert exit.value.args == (4,)

    else:
        main()

        options = intercepted_build_args.args[0]

        if platform == "linux":
            assert options.globals.architectures == {Architecture.i686}
        elif platform == "windows":
            assert options.globals.architectures == {Architecture.x86}


def test_archs_platform_all(
    platform: str, intercepted_build_args: ArgsInterceptor, monkeypatch: pytest.MonkeyPatch
) -> None:
    monkeypatch.setenv("CIBW_ARCHS", "all")

    main()
    options = intercepted_build_args.args[0]

    if platform == "linux":
        assert options.globals.architectures == {
            Architecture.x86_64,
            Architecture.i686,
            Architecture.aarch64,
            Architecture.ppc64le,
            Architecture.s390x,
            Architecture.armv7l,
            Architecture.riscv64,
        }
    elif platform == "windows":
        assert options.globals.architectures == {
            Architecture.x86,
            Architecture.AMD64,
            Architecture.ARM64,
        }
    elif platform == "macos":
        assert options.globals.architectures == {
            Architecture.x86_64,
            Architecture.arm64,
            Architecture.universal2,
        }


@pytest.mark.parametrize(
    ("only", "plat"),
    (
        ("cp311-manylinux_x86_64", "linux"),
        ("cp310-win_amd64", "windows"),
        ("cp310-win32", "windows"),
        ("cp311-macosx_x86_64", "macos"),
    ),
)
def test_only_argument(
    intercepted_build_args: ArgsInterceptor,
    monkeypatch: pytest.MonkeyPatch,
    only: str,
    plat: PlatformName,
) -> None:
    monkeypatch.setenv("CIBW_BUILD", "unused")
    monkeypatch.setenv("CIBW_SKIP", "unused")
    monkeypatch.setattr(sys, "argv", [*sys.argv, "--only", only])

    main()

    options = intercepted_build_args.args[0]
    assert options.globals.build_selector.build_config == only
    assert options.globals.build_selector.skip_config == ""
    assert options.platform == plat
    assert options.globals.architectures == Architecture.all_archs(plat)
    assert EnableGroup.PyPy in options.globals.build_selector.enable


@pytest.mark.parametrize("only", ("cp311-manylxinux_x86_64", "some_linux_thing"))
def test_only_failed(monkeypatch: pytest.MonkeyPatch, only: str) -> None:
    monkeypatch.setattr(sys, "argv", [*sys.argv, "--only", only])

    with pytest.raises(SystemExit):
        main()


def test_only_no_platform(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(
        sys, "argv", [*sys.argv, "--only", "cp311-manylinux_x86_64", "--platform", "macos"]
    )

    with pytest.raises(SystemExit):
        main()


def test_only_no_archs(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(
        sys, "argv", [*sys.argv, "--only", "cp311-manylinux_x86_64", "--archs", "x86_64"]
    )

    with pytest.raises(SystemExit):
        main()


@pytest.mark.parametrize(
    ("envvar_name", "envvar_value"),
    (
        ("CIBW_BUILD", "cp310-*"),
        ("CIBW_SKIP", "cp311-*"),
        ("CIBW_ARCHS", "auto32"),
        ("CIBW_PLATFORM", "macos"),
    ),
)
def test_only_overrides_env_vars(
    monkeypatch: pytest.MonkeyPatch,
    intercepted_build_args: ArgsInterceptor,
    envvar_name: str,
    envvar_value: str,
) -> None:
    monkeypatch.setattr(sys, "argv", [*sys.argv, "--only", "cp311-manylinux_x86_64"])
    monkeypatch.setenv(envvar_name, envvar_value)

    main()

    options = intercepted_build_args.args[0]
    assert options.globals.build_selector.build_config == "cp311-manylinux_x86_64"
    assert options.globals.build_selector.skip_config == ""
    assert options.platform == "linux"
    assert options.globals.architectures == Architecture.all_archs("linux")


def test_pyodide_on_windows(
    monkeypatch: pytest.MonkeyPatch, capsys: pytest.CaptureFixture[str]
) -> None:
    monkeypatch.setattr(sys, "platform", "win32")
    monkeypatch.setattr(sys, "argv", [*sys.argv, "--only", "cp312-pyodide_wasm32"])

    with pytest.raises(SystemExit) as exit:
        main()

    _, err = capsys.readouterr()

    assert exit.value.code == 2
    assert "Building for pyodide is not supported on Windows" in err


def test_empty_archs_platform(
    platform: PlatformName, intercepted_build_args: ArgsInterceptor, monkeypatch: pytest.MonkeyPatch
) -> None:
    monkeypatch.setenv("CIBW_ARCHS", "")

    main()

    options = intercepted_build_args.args[0]
    assert options.globals.architectures == Architecture.auto_archs(platform)



import sys
import sysconfig
from pathlib import Path

assert sys.platform == "android"
assert sysconfig.get_platform().startswith("android-")

android_prefix = Path(f"{sys.prefix}/../python/prefix").resolve()
assert android_prefix.is_dir()

vars = sysconfig.get_config_vars()
assert vars["INCLUDEDIR"] == f"{android_prefix}/include"
assert vars["LDVERSION"] == f"{sys.version_info[0]}.{sys.version_info[1]}{sys.abiflags}"
assert vars["INCLUDEPY"] == f"{vars['INCLUDEDIR']}/python{vars['LDVERSION']}"
assert vars["LIBDIR"] == f"{android_prefix}/lib"
assert vars["Py_ENABLE_SHARED"] == 1

paths = sysconfig.get_paths()
assert paths["include"] == vars["INCLUDEPY"]
assert paths["platinclude"] == vars["INCLUDEPY"]



import contextlib
import functools
import os
import shutil
import sys
import tomllib
from collections.abc import Sequence
from pathlib import Path
from typing import Final

from filelock import FileLock
from packaging.requirements import InvalidRequirement, Requirement
from packaging.version import Version

from cibuildwheel.util import resources
from cibuildwheel.util.cmd import call
from cibuildwheel.util.file import CIBW_CACHE_PATH, download

_IS_WIN: Final[bool] = sys.platform.startswith("win")


@functools.cache
def _ensure_virtualenv(version: str) -> tuple[Path, Version]:
    version_parts = version.split(".")
    key = f"py{version_parts[0]}{version_parts[1]}"
    with resources.VIRTUALENV.open("rb") as f:
        loaded_file = tomllib.load(f)
    configuration = loaded_file.get(key, loaded_file["default"])
    version = str(configuration["version"])
    url = str(configuration["url"])
    path = CIBW_CACHE_PATH / f"virtualenv-{version}.pyz"
    with FileLock(str(path) + ".lock"):
        if not path.exists():
            download(url, path)
    return (path, Version(version))


def constraint_flags(
    dependency_constraint: Path | None,
) -> Sequence[str]:
    """
    Returns the flags to pass to pip for the given dependency constraint.
    """

    return ["-c", dependency_constraint.as_uri()] if dependency_constraint else []


def _parse_pip_constraint_for_virtualenv(
    constraint_path: Path | None,
) -> str:
    """
    Parses the constraints file referenced by `dependency_constraint_flags` and returns a dict where
    the key is the package name, and the value is the constraint version.
    If a package version cannot be found, its value is "embed" meaning that virtualenv will install
    its bundled version, already available locally.
    The function does not try to be too smart and just handles basic constraints.
    If it can't get an exact version, the real constraint will be handled by the
    {macos|windows}.setup_python function.
    """
    if constraint_path:
        assert constraint_path.exists()
        with constraint_path.open(encoding="utf-8") as constraint_file:
            for line_ in constraint_file:
                line = line_.strip()
                if not line:
                    continue
                if line.startswith("#"):
                    continue
                try:
                    requirement = Requirement(line)
                    package = requirement.name
                    if (
                        package != "pip"
                        or requirement.url is not None
                        or requirement.marker is not None
                        or len(requirement.extras) != 0
                        or len(requirement.specifier) != 1
                    ):
                        continue
                    specifier = next(iter(requirement.specifier))
                    if specifier.operator != "==":
                        continue
                    return specifier.version
                except InvalidRequirement:
                    continue
    return "embed"


def virtualenv(
    version: str,
    python: Path,
    venv_path: Path,
    dependency_constraint: Path | None,
    *,
    use_uv: bool,
    env: dict[str, str] | None = None,
    pip_version: str | None = None,
) -> dict[str, str]:
    """
    Create a virtual environment. If `use_uv` is True,
    dependency_constraint_flags are ignored since nothing is installed in the
    venv. Otherwise, pip is installed.
    """

    # virtualenv may fail if this is a symlink.
    python = python.resolve()

    assert python.exists()

    if use_uv:
        uv_path = find_uv()
        assert uv_path is not None
        call(uv_path, "venv", venv_path, "--python", python)
    else:
        virtualenv_app, virtualenv_version = _ensure_virtualenv(version)
        if pip_version is None:
            pip_version = _parse_pip_constraint_for_virtualenv(dependency_constraint)
        additional_flags = [f"--pip={pip_version}", "--no-setuptools"]
        if virtualenv_version < Version("20.31") or Version(version) < Version("3.9"):
            additional_flags.append("--no-wheel")

        # Using symlinks to pre-installed seed packages is really the fastest way to get a virtual
        # environment. The initial cost is a bit higher but reusing is much faster.
        # Windows does not always allow symlinks so just disabling for now.
        # Requires pip>=19.3 so disabling for "embed" because this means we don't know what's the
        # version of pip that will end-up installed.
        # c.f. https://virtualenv.pypa.io/en/latest/cli_interface.html#section-seeder
        if not _IS_WIN and pip_version != "embed" and Version(pip_version) >= Version("19.3"):
            additional_flags.append("--symlink-app-data")

        call(
            sys.executable,
            "-sS",  # just the stdlib, https://github.com/pypa/virtualenv/issues/2133#issuecomment-1003710125
            virtualenv_app,
            "--activators=",
            "--no-periodic-update",
            *additional_flags,
            "--python",
            python,
            venv_path,
        )
    paths = [str(venv_path), str(venv_path / "Scripts")] if _IS_WIN else [str(venv_path / "bin")]
    venv_env = os.environ.copy() if env is None else env.copy()
    venv_env["PATH"] = os.pathsep.join([*paths, venv_env["PATH"]])
    venv_env["VIRTUAL_ENV"] = str(venv_path)
    if not use_uv and pip_version == "embed":
        call(
            "python",
            "-m",
            "pip",
            "install",
            "--upgrade",
            "pip",
            *constraint_flags(dependency_constraint),
            env=venv_env,
            cwd=venv_path,
        )
    return venv_env


def find_uv() -> Path | None:
    # Prefer uv in our environment
    with contextlib.suppress(ImportError, FileNotFoundError):
        from uv import find_uv_bin  # noqa: PLC0415

        return Path(find_uv_bin())

    uv_on_path = shutil.which("uv")
    return Path(uv_on_path) if uv_on_path else None



#!/usr/bin/env python3

import json

from cibuildwheel.extra import github_api_request
from cibuildwheel.util.python_build_standalone import (
    PythonBuildStandaloneAsset,
    PythonBuildStandaloneReleaseData,
)
from cibuildwheel.util.resources import PYTHON_BUILD_STANDALONE_RELEASES


def main() -> None:
    """
    This script updates the vendored list of release assets to the latest
    version of astral-sh/python-build-standalone.
    """

    # Get the latest release tag from the GitHub API
    latest_release = github_api_request("repos/astral-sh/python-build-standalone/releases/latest")
    latest_tag = latest_release["tag_name"]

    # Get the list of assets for the latest release
    github_assets = github_api_request(
        f"repos/astral-sh/python-build-standalone/releases/tags/{latest_tag}"
    )["assets"]

    assets = [
        PythonBuildStandaloneAsset(name=ga["name"], url=ga["browser_download_url"])
        for ga in github_assets
        if ga["name"].endswith("install_only.tar.gz")
    ]

    # Try to keep output order stable
    assets = sorted(assets, key=lambda x: x["name"])

    # Write the assets to the JSON file. One day, we might need to support
    # multiple releases, but for now, we only support the latest one
    json_file_contents = PythonBuildStandaloneReleaseData(
        releases=[
            {
                "tag": latest_tag,
                "assets": assets,
            }
        ]
    )

    with PYTHON_BUILD_STANDALONE_RELEASES.open("w", encoding="utf-8") as f:
        json.dump(json_file_contents, f, indent=2)
        # Add a trailing newline, our pre-commit hook requires it
        f.write("\n")

    print(
        f"Updated {PYTHON_BUILD_STANDALONE_RELEASES.name} with {len(assets)} assets for tag {latest_tag}"
    )


if __name__ == "__main__":
    main()



import subprocess
import textwrap
from pathlib import Path

import pytest

from . import test_projects, utils

# pyodide does not support building without isolation, need to check the base_prefix
SYS_PREFIX = f"sys.{'base_' if utils.get_platform() == 'pyodide' else ''}prefix"


project_with_before_build_asserts = test_projects.new_c_project(
    setup_py_add=textwrap.dedent(
        rf"""
        import os

        # assert that the Python version as written to pythonversion_bb.txt in the CIBW_BEFORE_BUILD step
        # is the same one as is currently running.
        with open('pythonversion_bb.txt') as f:
            stored_version = f.read()
        print('stored_version', stored_version)
        print('sys.version', sys.version)
        assert stored_version == sys.version

        # check that the prefix also was written
        with open('pythonprefix_bb.txt') as f:
            stored_prefix = f.read()
        print('stored_prefix', stored_prefix)
        print('{SYS_PREFIX}', {SYS_PREFIX})
        #  Works around path-comparison bugs caused by short-paths on Windows e.g.
        #  vssadm~1 instead of vssadministrator

        assert os.path.samefile(stored_prefix, {SYS_PREFIX})
        """
    )
)


def test(tmp_path: Path) -> None:
    project_dir = tmp_path / "project"
    project_with_before_build_asserts.generate(project_dir)

    before_build = (
        """python -c "import pathlib, sys; pathlib.Path('{project}/pythonversion_bb.txt').write_text(sys.version)" && """
        f'''python -c "import pathlib, sys; pathlib.Path('{{project}}/pythonprefix_bb.txt').write_text({SYS_PREFIX})"'''
    )
    frontend = "build"
    if utils.get_platform() != "pyodide":
        before_build = f"python -m pip install setuptools && {before_build}"
        frontend = f"{frontend};args: --no-isolation"

    # build the wheels
    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={
            # write python version information to a temporary file, this is
            # checked in setup.py
            "CIBW_BEFORE_BUILD": before_build,
            "CIBW_BUILD_FRONTEND": frontend,
        },
    )

    # also check that we got the right wheels
    expected_wheels = utils.expected_wheels("spam", "0.1.0")
    assert set(actual_wheels) == set(expected_wheels)


def test_failing_command(tmp_path: Path) -> None:
    project_dir = tmp_path / "project"
    test_projects.new_c_project().generate(project_dir)

    with pytest.raises(subprocess.CalledProcessError):
        utils.cibuildwheel_run(
            project_dir,
            add_env={
                "CIBW_BEFORE_BUILD": "false",
                "CIBW_BEFORE_BUILD_WINDOWS": "exit /b 1",
            },
        )


def test_cwd(tmp_path: Path) -> None:
    project_dir = tmp_path / "project"
    test_projects.new_c_project().generate(project_dir)

    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_env={
            "CIBW_BEFORE_BUILD": f'''python -c "import os; assert os.getcwd() == {str(project_dir)!r}"''',
            "CIBW_BEFORE_BUILD_LINUX": '''python -c "import os; assert os.getcwd() == '/project'"''',
        },
        single_python=True,
    )

    expected_wheels = utils.expected_wheels("spam", "0.1.0", single_python=True)
    assert set(actual_wheels) == set(expected_wheels)



"""
Utility functions used by the cibuildwheel tests.

This file is added to the PYTHONPATH in the test runner at bin/run_test.py.
"""

import os
import platform as pm
import subprocess
import sys
from collections.abc import Generator, Mapping, Sequence
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Final

import pytest

from cibuildwheel.architecture import Architecture
from cibuildwheel.ci import CIProvider, detect_ci_provider
from cibuildwheel.selector import EnableGroup
from cibuildwheel.util.file import CIBW_CACHE_PATH

EMULATED_ARCHS: Final[list[str]] = sorted(
    arch.value for arch in (Architecture.all_archs("linux") - Architecture.auto_archs("linux"))
)
PYPY_ARCHS = ["x86_64", "i686", "AMD64", "aarch64", "arm64"]
GRAALPY_ARCHS = ["x86_64", "AMD64", "aarch64", "arm64"]

SINGLE_PYTHON_VERSION: Final[tuple[int, int]] = (3, 12)

# temporary workaround: set by build_frontend_env fixture to skip graalpy
# when uv is the build frontend (compatibility issue between graalpy and uv)
include_graalpy_in_expected_wheels: bool = True

_AARCH64_CAN_RUN_ARMV7: Final[bool] = Architecture.aarch64.value not in EMULATED_ARCHS and {
    None: Architecture.armv7l.value not in EMULATED_ARCHS,
    CIProvider.travis_ci: False,
    CIProvider.cirrus_ci: False,
}.get(detect_ci_provider(), True)


def get_platform() -> str:
    """Return the current platform as determined by CIBW_PLATFORM or sys.platform."""
    platform = os.environ.get("CIBW_PLATFORM", "")
    if platform:
        return platform
    elif sys.platform.startswith("linux"):
        return "linux"
    elif sys.platform.startswith("darwin"):
        return "macos"
    elif sys.platform.startswith(("win32", "cygwin")):
        return "windows"
    else:
        msg = f"Unsupported platform {sys.platform!r}"
        raise Exception(msg)


DEFAULT_CIBW_ENABLE = "cpython-freethreading cpython-prerelease"


def get_enable_groups() -> frozenset[EnableGroup]:
    value = os.environ.get("CIBW_ENABLE", DEFAULT_CIBW_ENABLE)
    return EnableGroup.parse_option_value(value)


def cibuildwheel_get_build_identifiers(
    project_path: Path,
    env: dict[str, str] | None = None,
) -> list[str]:
    """
    Returns the list of build identifiers that cibuildwheel will try to build
    for the current platform.
    """
    cmd = [sys.executable, "-m", "cibuildwheel", "--print-build-identifiers", str(project_path)]
    if env is None:
        env = os.environ.copy()

    cmd_output = subprocess.run(
        cmd,
        text=True,
        env=env,
        check=True,
        stdout=subprocess.PIPE,
    ).stdout

    return cmd_output.strip().split("\n")


def _update_pip_cache_dir(env: dict[str, str]) -> None:
    # Fix for pip concurrency bug https://github.com/pypa/pip/issues/11340
    # See https://github.com/pypa/cibuildwheel/issues/1254 for discussion.
    if get_platform() == "linux":
        return
    if "PIP_CACHE_DIR" in env:
        return
    worker_id = os.environ.get("PYTEST_XDIST_WORKER")
    if worker_id is None or worker_id == "gw0":
        return
    pip_cache_dir = CIBW_CACHE_PATH / "test_cache" / f"pip_cache_dir_{worker_id}"
    env["PIP_CACHE_DIR"] = str(pip_cache_dir)


def cibuildwheel_run(
    project_path: str | Path,
    package_dir: str | Path = ".",
    env: dict[str, str] | None = None,
    add_env: Mapping[str, str] | None = None,
    output_dir: Path | None = None,
    add_args: Sequence[str] | None = None,
    single_python: bool = False,
) -> list[str]:
    """
    Runs cibuildwheel as a subprocess, building the project at project_path.

    Uses the current Python interpreter.

    :param project_path: path of the project to be built.
    :param package_dir: path of the package to be built. Can be absolute, or
    relative to project_path.
    :param env: full environment to be used, os.environ if None
    :param add_env: environment used to update env
    :param output_dir: directory where wheels are saved. If None, a temporary
    directory will be used for the duration of the command.
    :param add_args: Additional command-line arguments to pass to cibuildwheel.
    :return: list of built wheels (file names).
    """
    if env is None:
        env = os.environ.copy()
        # If present in the host environment, remove the MACOSX_DEPLOYMENT_TARGET for consistency
        env.pop("MACOSX_DEPLOYMENT_TARGET", None)

    if add_args is None:
        add_args = []

    if add_env is not None:
        env.update(add_env)

    _update_pip_cache_dir(env)

    if single_python:
        env["CIBW_BUILD"] = "cp{}{}-*".format(*SINGLE_PYTHON_VERSION)

    with TemporaryDirectory() as tmp_output_dir:
        subprocess.run(
            [
                sys.executable,
                "-m",
                "cibuildwheel",
                "--output-dir",
                str(output_dir or tmp_output_dir),
                str(package_dir),
                *add_args,
            ],
            env=env,
            cwd=project_path,
            check=True,
        )
        wheels = sorted(p.name for p in (output_dir or Path(tmp_output_dir)).iterdir())
    return wheels


def _floor_macosx(*args: str) -> str:
    """
    Make sure a deployment target is not less than some value.
    """
    return max(args, key=lambda x: tuple(map(int, x.split("."))))


def expected_wheels(
    package_name: str,
    package_version: str,
    manylinux_versions: list[str] | None = None,
    musllinux_versions: list[str] | None = None,
    macosx_deployment_target: str | None = None,
    iphoneos_deployment_target: str | None = None,
    machine_arch: str | None = None,
    platform: str | None = None,
    python_abi_tags: list[str] | None = None,
    include_universal2: bool = False,
    single_python: bool = False,
    single_arch: bool = False,
    full_auto: bool = False,
) -> list[str]:
    """
    Returns the expected wheels from a run of cibuildwheel.
    """
    platform = platform or get_platform()

    if machine_arch is None:
        machine_arch = pm.machine()
        if platform == "linux":
            machine_arch = arch_name_for_linux(machine_arch)

    if macosx_deployment_target is None:
        macosx_deployment_target = os.environ.get("MACOSX_DEPLOYMENT_TARGET", "10.9")

    if iphoneos_deployment_target is None:
        iphoneos_deployment_target = os.environ.get("IPHONEOS_DEPLOYMENT_TARGET", "13.0")

    architectures = [machine_arch]
    if not single_arch:
        if platform == "linux" and full_auto:
            if machine_arch == "x86_64":
                architectures.append("i686")
            elif (
                machine_arch == "aarch64"
                and sys.platform.startswith("linux")
                and _AARCH64_CAN_RUN_ARMV7
            ):
                architectures.append("armv7l")
        elif platform == "windows" and machine_arch == "AMD64":
            architectures.append("x86")

    return [
        wheel
        for architecture in architectures
        for wheel in _expected_wheels(
            package_name=package_name,
            package_version=package_version,
            machine_arch=architecture,
            manylinux_versions=manylinux_versions,
            musllinux_versions=musllinux_versions,
            macosx_deployment_target=macosx_deployment_target,
            iphoneos_deployment_target=iphoneos_deployment_target,
            platform=platform,
            python_abi_tags=python_abi_tags,
            include_universal2=include_universal2,
            single_python=single_python,
        )
    ]


def _expected_wheels(
    package_name: str,
    package_version: str,
    machine_arch: str,
    manylinux_versions: list[str] | None,
    musllinux_versions: list[str] | None,
    macosx_deployment_target: str,
    iphoneos_deployment_target: str,
    platform: str,
    python_abi_tags: list[str] | None,
    include_universal2: bool,
    single_python: bool,
) -> Generator[str, None, None]:
    """
    Returns a list of expected wheels from a run of cibuildwheel.
    """
    # per PEP 425 (https://www.python.org/dev/peps/pep-0425/), wheel files shall have name of the form
    # {distribution}-{version}(-{build tag})?-{python tag}-{abi tag}-{platform tag}.whl
    # {python tag} and {abi tag} are closely related to the python interpreter used to build the wheel
    # so we'll merge them below as python_abi_tag

    enable_groups = EnableGroup.parse_option_value(os.environ.get("CIBW_ENABLE", ""))

    if manylinux_versions is None:
        manylinux_versions = {
            "armv7l": ["manylinux2014", "manylinux_2_17", "manylinux_2_31"],
            "i686": ["manylinux1", "manylinux_2_28", "manylinux_2_5"],
            "x86_64": ["manylinux1", "manylinux_2_28", "manylinux_2_5"],
            "riscv64": ["manylinux_2_31", "manylinux_2_39"],
        }.get(machine_arch, ["manylinux2014", "manylinux_2_17", "manylinux_2_28"])

    if musllinux_versions is None:
        musllinux_versions = ["musllinux_1_2"]

    if platform == "pyodide" and python_abi_tags is None:
        python_abi_tags = [
            "cp312-cp312",
            "cp313-cp313",
        ]
    elif platform == "android" and python_abi_tags is None:  # noqa: SIM114
        python_abi_tags = [
            "cp313-cp313",
            "cp314-cp314",
        ]
    elif platform == "ios" and python_abi_tags is None:
        python_abi_tags = [
            "cp313-cp313",
            "cp314-cp314",
        ]
    elif python_abi_tags is None:
        python_abi_tags = [
            "cp38-cp38",
            "cp39-cp39",
            "cp310-cp310",
            "cp311-cp311",
            "cp312-cp312",
            "cp313-cp313",
            "cp314-cp314",
            "cp314-cp314t",
        ]

        enable_groups = get_enable_groups()
        if EnableGroup.CPythonFreeThreading in enable_groups:
            python_abi_tags.append("cp313-cp313t")

        if EnableGroup.CPythonPrerelease in enable_groups:
            ...  # Add cp315 here when available

        if EnableGroup.PyPyEoL in enable_groups:
            python_abi_tags += [
                "pp38-pypy38_pp73",
                "pp39-pypy39_pp73",
                "pp310-pypy310_pp73",
            ]
        if EnableGroup.PyPy in enable_groups:
            python_abi_tags += [
                "pp311-pypy311_pp73",
            ]

        if EnableGroup.GraalPy in enable_groups and include_graalpy_in_expected_wheels:
            python_abi_tags += [
                "graalpy311-graalpy242_311_native",
                "graalpy312-graalpy250_312_native",
            ]

    if machine_arch == "ARM64" and platform == "windows":
        # no CPython 3.8 on Windows ARM64
        python_abi_tags = [t for t in python_abi_tags if not t.startswith("cp38")]

    if machine_arch not in PYPY_ARCHS:
        python_abi_tags = [tag for tag in python_abi_tags if not tag.startswith("pp")]

    if machine_arch not in GRAALPY_ARCHS:
        python_abi_tags = [tag for tag in python_abi_tags if not tag.startswith("graalpy")]

    if single_python:
        python_tag = "cp{}{}-".format(*SINGLE_PYTHON_VERSION)
        python_abi_tags = [
            next(
                tag
                for tag in python_abi_tags
                if tag.startswith(python_tag) and not tag.endswith("t")
            )
        ]

    for python_abi_tag in python_abi_tags:
        platform_tags = []

        if platform == "linux":
            if len(manylinux_versions) > 0:
                platform_tags = [
                    ".".join(
                        f"{manylinux_version}_{machine_arch}"
                        for manylinux_version in manylinux_versions
                    )
                ]
            if len(musllinux_versions) > 0 and not python_abi_tag.startswith(("pp", "graalpy")):
                platform_tags.append(
                    ".".join(
                        f"{musllinux_version}_{machine_arch}"
                        for musllinux_version in musllinux_versions
                    )
                )

        elif platform == "windows":
            platform_tags = {
                "AMD64": ["win_amd64"],
                "ARM64": ["win_arm64"],
                "x86": ["win32"],
            }.get(machine_arch, [])

        elif platform == "macos":
            if python_abi_tag.startswith("pp"):
                if python_abi_tag.startswith("pp38"):
                    min_macosx = macosx_deployment_target
                else:
                    min_macosx = _floor_macosx(macosx_deployment_target, "10.15")
            elif python_abi_tag.startswith("cp"):
                if python_abi_tag.startswith(("cp38", "cp39", "cp310", "cp311")):
                    min_macosx = macosx_deployment_target
                elif python_abi_tag.startswith(("cp312", "cp313")):
                    min_macosx = _floor_macosx(macosx_deployment_target, "10.13")
                else:
                    min_macosx = _floor_macosx(macosx_deployment_target, "10.15")
            elif python_abi_tag.startswith("graalpy"):
                if python_abi_tag.startswith("graalpy311"):
                    min_macosx = macosx_deployment_target
                else:
                    min_macosx = _floor_macosx(macosx_deployment_target, "10.13")
            else:
                min_macosx = macosx_deployment_target

            if machine_arch == "arm64":
                arm64_macosx = _floor_macosx(min_macosx, "11.0")
                platform_tags = [f"macosx_{arm64_macosx.replace('.', '_')}_arm64"]
            else:
                platform_tags = [f"macosx_{min_macosx.replace('.', '_')}_x86_64"]

            if include_universal2:
                platform_tags.append(f"macosx_{min_macosx.replace('.', '_')}_universal2")

        elif platform == "android":
            api_level = {
                "cp313-cp313": 21,
                "cp314-cp314": 24,
            }[python_abi_tag]
            platform_tags = [f"android_{api_level}_{machine_arch}"]

        elif platform == "ios":
            if machine_arch == "x86_64":
                platform_tags = [
                    f"ios_{iphoneos_deployment_target.replace('.', '_')}_x86_64_iphonesimulator"
                ]
            elif machine_arch == "arm64":
                platform_tags = [
                    f"ios_{iphoneos_deployment_target.replace('.', '_')}_arm64_iphoneos",
                    f"ios_{iphoneos_deployment_target.replace('.', '_')}_arm64_iphonesimulator",
                ]
            else:
                msg = f"Unsupported architecture {machine_arch!r} for iOS"
                raise Exception(msg)

        elif platform == "pyodide":
            platform_tags = {
                "cp312-cp312": ["pyodide_2024_0_wasm32"],
                "cp313-cp313": ["pyodide_2025_0_wasm32"],
            }.get(python_abi_tag, [])

            if not platform_tags:
                # for example if the python tag is `none` or `abi3`, all
                # platform tags are built with that python tag
                platform_tags = ["pyodide_2024_0_wasm32"]

        else:
            msg = f"Unsupported platform {platform!r}"
            raise Exception(msg)

        for platform_tag in platform_tags:
            yield f"{package_name}-{package_version}-{python_abi_tag}-{platform_tag}.whl"


def get_macos_version() -> tuple[int, int]:
    """
    Returns the macOS major/minor version, as a tuple, e.g. (10, 15) or (11, 0)

    These tuples can be used in comparisons, e.g.
        (10, 14) <= (11, 0) == True
        (11, 2) <= (11, 0) != True
    """
    version_str, _, _ = pm.mac_ver()
    return tuple(map(int, version_str.split(".")[:2]))  # type: ignore[return-value]


def get_xcode_version() -> tuple[int, int]:
    """Calls `xcodebuild -version` to retrieve the Xcode version as a 2-tuple."""
    output = subprocess.run(
        ["xcodebuild", "-version"],
        text=True,
        check=True,
        stdout=subprocess.PIPE,
    ).stdout
    lines = output.splitlines()
    _, version_str = lines[0].split()

    version_parts = version_str.split(".")
    return (int(version_parts[0]), int(version_parts[1]))


def skip_if_pyodide(reason: str) -> pytest.MarkDecorator:
    return pytest.mark.skipif(get_platform() == "pyodide", reason=reason)


def invoke_pytest() -> str:
    # see https://github.com/pyodide/pyodide/issues/4802
    if get_platform() == "pyodide":
        return "python -m pytest"
    return "pytest"


def arch_name_for_linux(arch: str) -> str:
    """
    Archs have different names on different platforms, but it's useful to be
    able to run linux tests on dev machines. This function translates between
    the different names.
    """
    if arch == "arm64":
        return "aarch64"
    return arch



"""
These are utilities for the `/bin` scripts, not for the `cibuildwheel` program.
"""

import json
import time
import typing
import urllib.error
import urllib.request
from collections.abc import Mapping, Sequence
from io import StringIO
from typing import Any, NotRequired, Protocol

from cibuildwheel import __version__ as cibw_version

__all__ = ("Printable", "dump_python_configurations")


class Printable(Protocol):
    def __str__(self) -> str: ...


def dump_python_configurations(
    inp: Mapping[str, Mapping[str, Sequence[Mapping[str, Printable]]]],
) -> str:
    output = StringIO()
    for header, values in inp.items():
        output.write(f"[{header}]\n")
        for inner_header, listing in values.items():
            output.write(f"{inner_header} = [\n")
            for item in listing:
                output.write("  { ")
                dict_contents = (f'{key} = "{value}"' for key, value in item.items())
                output.write(", ".join(dict_contents))
                output.write(" },\n")
            output.write("]\n")
        output.write("\n")
    # Strip the final newline, to avoid two blank lines at the end.
    return output.getvalue()[:-1]


def _json_request(request: urllib.request.Request, timeout: int = 30) -> dict[str, Any]:
    with urllib.request.urlopen(request, timeout=timeout) as response:
        return typing.cast("dict[str, Any]", json.load(response))


def github_api_request(path: str, *, max_retries: int = 3) -> dict[str, Any]:
    """
    Makes a GitHub API request to the given path and returns the JSON response.
    """
    api_url = f"https://api.github.com/{path}"
    headers = {
        "Accept": "application/vnd.github.v3+json",
        "User-Agent": f"cibuildwheel/{cibw_version}",
    }
    request = urllib.request.Request(api_url, headers=headers)

    for retry_count in range(max_retries):
        try:
            return _json_request(request)
        except (urllib.error.URLError, TimeoutError) as e:
            # pylint: disable=E1101
            if (
                isinstance(e, urllib.error.HTTPError)
                and (e.code in {403, 429})
                and e.headers.get("x-ratelimit-remaining") == "0"
            ):
                reset_time = int(e.headers.get("x-ratelimit-reset", 0))
                wait_time = max(0, reset_time - int(e.headers.get("date", 0)))
                print(f"Github rate limit exceeded. Waiting for {wait_time} seconds.")
                time.sleep(wait_time)
            else:
                print(f"Retrying GitHub API request due to error: {e}")

            if retry_count == max_retries - 1:
                print(f"GitHub API request failed (Network error: {e}). Check network connection.")
                raise e

    # Should never be reached but to keep the type checker happy
    msg = "Unexpected execution path in github_api_request"
    raise RuntimeError(msg)


class PyodideXBuildEnvRelease(typing.TypedDict):
    version: str
    python_version: str
    emscripten_version: str
    min_pyodide_build_version: NotRequired[str]
    max_pyodide_build_version: NotRequired[str]


class PyodideXBuildEnvInfo(typing.TypedDict):
    releases: dict[str, PyodideXBuildEnvRelease]


def get_pyodide_xbuildenv_info() -> PyodideXBuildEnvInfo:
    xbuildenv_info_url = (
        "https://pyodide.github.io/pyodide/api/pyodide-cross-build-environments.json"
    )
    with urllib.request.urlopen(xbuildenv_info_url) as response:
        return typing.cast("PyodideXBuildEnvInfo", json.loads(response.read().decode("utf-8")))



from pathlib import Path

import pytest

from cibuildwheel.util.packaging import DependencyConstraints


def test_defaults(tmp_path: Path) -> None:
    dependency_constraints = DependencyConstraints.pinned()

    project_root = Path(__file__).parents[1]
    resources_dir = project_root / "cibuildwheel" / "resources"

    assert dependency_constraints.base_file_path
    assert dependency_constraints.base_file_path.samefile(resources_dir / "constraints.txt")

    constraints_file = dependency_constraints.get_for_python_version(
        version="3.99", tmp_dir=tmp_path
    )
    assert constraints_file
    assert constraints_file.samefile(resources_dir / "constraints.txt")

    constraints_file = dependency_constraints.get_for_python_version(
        version="3.9", tmp_dir=tmp_path
    )
    assert constraints_file
    assert constraints_file.samefile(resources_dir / "constraints-python39.txt")

    constraints_file = dependency_constraints.get_for_python_version(
        version="3.13", tmp_dir=tmp_path
    )
    assert constraints_file
    assert constraints_file.samefile(resources_dir / "constraints-python313.txt")


def test_inline_packages(tmp_path: Path) -> None:
    dependency_constraints = DependencyConstraints(
        base_file_path=None,
        packages=["foo==1.2.3", "bar==4.5.6"],
    )

    constraint_file = dependency_constraints.get_for_python_version(version="x.x", tmp_dir=tmp_path)
    assert constraint_file
    constraints_file_contents = constraint_file.read_text()

    assert constraints_file_contents == "foo==1.2.3\nbar==4.5.6"


@pytest.mark.parametrize("config_string", ["", "latest", "packages:"])
def test_empty_constraints(config_string: str) -> None:
    dependency_constraints = DependencyConstraints.from_config_string(config_string)

    assert not dependency_constraints.packages
    assert not dependency_constraints.base_file_path
    assert dependency_constraints == DependencyConstraints.latest()



import shlex
from pathlib import Path
from typing import Any, cast

import pytest

from cibuildwheel.options import (
    EnvironmentFormat,
    InheritRule,
    ListFormat,
    OptionsReader,
    OptionsReaderError,
    ShlexTableFormat,
    _resolve_cascade,
)
from cibuildwheel.typing import PlatformName

PYPROJECT_1 = """
[tool.cibuildwheel]
build = "cp39*"
environment = {THING = "OTHER", FOO="BAR"}
xbuild-tools = ["first"]

test-command = "pyproject"
test-requires = "something"
test-extras = ["one", "two"]
test-groups = ["three", "four"]
test-sources = ["five", "six and seven"]

manylinux-x86_64-image = "manylinux_2_28"

[tool.cibuildwheel.macos]
test-requires = "else"

[tool.cibuildwheel.linux]
test-requires = ["other", "many"]
"""


@pytest.fixture(params=["linux", "macos", "windows"])
def platform(request: pytest.FixtureRequest) -> PlatformName:
    return cast("PlatformName", request.param)


@pytest.mark.parametrize("fname", ["pyproject.toml", "cibuildwheel.toml"])
def test_simple_settings(tmp_path: Path, platform: PlatformName, fname: str) -> None:
    config_file_path: Path = tmp_path / fname
    config_file_path.write_text(PYPROJECT_1)

    options_reader = OptionsReader(config_file_path, platform=platform, env={})

    assert options_reader.get("build", option_format=ListFormat(" "), env_plat=False) == "cp39*"

    assert options_reader.get("test-command") == "pyproject"
    assert options_reader.get("archs", option_format=ListFormat(" ")) == "auto"
    assert (
        options_reader.get("test-sources", option_format=ListFormat(" ", quote=shlex.quote))
        == "five 'six and seven'"
    )
    assert (
        options_reader.get("test-requires", option_format=ListFormat(" "))
        == {"windows": "something", "macos": "else", "linux": "other many"}[platform]
    )

    # Also testing options for support for both lists and tables
    assert (
        options_reader.get("environment", option_format=EnvironmentFormat())
        == 'THING="OTHER" FOO="BAR"'
    )
    assert options_reader.get("test-extras", option_format=ListFormat(",")) == "one,two"
    assert options_reader.get("test-groups", option_format=ListFormat(" ")) == "three four"

    assert options_reader.get("manylinux-x86_64-image") == "manylinux_2_28"
    assert options_reader.get("manylinux-i686-image") == "manylinux_2_28"

    with pytest.raises(OptionsReaderError):
        # fails because the option is a table and the option_format only works with lists
        options_reader.get("environment", option_format=ListFormat(" "))

    with pytest.raises(OptionsReaderError):
        # fails because the option is a list and the option_format only works with tables
        options_reader.get("test-extras", option_format=ShlexTableFormat())


def test_envvar_override(tmp_path: Path, platform: PlatformName) -> None:
    config_file_path: Path = tmp_path / "pyproject.toml"
    config_file_path.write_text(PYPROJECT_1)

    options_reader = OptionsReader(
        config_file_path,
        platform=platform,
        env={
            "CIBW_BUILD": "cp38*",
            "CIBW_MANYLINUX_X86_64_IMAGE": "manylinux_2_24",
            "CIBW_XBUILD_TOOLS": "cmake rustc",
            "CIBW_TEST_COMMAND": "mytest",
            "CIBW_TEST_REQUIRES": "docs",
            "CIBW_TEST_GROUPS": "mgroup two",
            "CIBW_TEST_REQUIRES_LINUX": "scod",
            "CIBW_TEST_GROUPS_LINUX": "lgroup",
            "CIBW_TEST_SOURCES": 'first "second third"',
        },
    )

    assert options_reader.get("archs", option_format=ListFormat(" ")) == "auto"

    assert options_reader.get("build") == "cp38*"
    assert options_reader.get("manylinux-x86_64-image") == "manylinux_2_24"
    assert options_reader.get("manylinux-i686-image") == "manylinux_2_28"

    assert (
        options_reader.get("xbuild-tools", option_format=ListFormat(" ", quote=shlex.quote))
        == "cmake rustc"
    )
    assert (
        options_reader.get("test-sources", option_format=ListFormat(" ", quote=shlex.quote))
        == 'first "second third"'
    )
    assert (
        options_reader.get("test-requires", option_format=ListFormat(" "))
        == {"windows": "docs", "macos": "docs", "linux": "scod"}[platform]
    )
    assert (
        options_reader.get("test-groups", option_format=ListFormat(" "))
        == {"windows": "mgroup two", "macos": "mgroup two", "linux": "lgroup"}[platform]
    )
    assert options_reader.get("test-command") == "mytest"


def test_project_global_override_default_platform(tmp_path: Path, platform: PlatformName) -> None:
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel]
repair-wheel-command = "repair-project-global"
"""
    )
    options_reader = OptionsReader(pyproject_toml, platform=platform, env={})
    assert options_reader.get("repair-wheel-command") == "repair-project-global"


def test_env_global_override_default_platform(platform: PlatformName) -> None:
    options_reader = OptionsReader(
        platform=platform, env={"CIBW_REPAIR_WHEEL_COMMAND": "repair-env-global"}
    )
    assert options_reader.get("repair-wheel-command") == "repair-env-global"


def test_env_global_override_project_platform(tmp_path: Path, platform: PlatformName) -> None:
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel.linux]
repair-wheel-command = "repair-project-linux"
[tool.cibuildwheel.windows]
repair-wheel-command = "repair-project-windows"
[tool.cibuildwheel.macos]
repair-wheel-command = "repair-project-macos"
"""
    )
    options_reader = OptionsReader(
        pyproject_toml,
        platform=cast("Any", platform),
        env={
            "CIBW_REPAIR_WHEEL_COMMAND": "repair-env-global",
        },
    )
    assert options_reader.get("repair-wheel-command") == "repair-env-global"


def test_global_platform_order(tmp_path: Path, platform: str) -> None:
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel.linux]
repair-wheel-command = "repair-project-linux"
[tool.cibuildwheel.windows]
repair-wheel-command = "repair-project-windows"
[tool.cibuildwheel.macos]
repair-wheel-command = "repair-project-macos"
[tool.cibuildwheel]
repair-wheel-command = "repair-project-global"
"""
    )
    options_reader = OptionsReader(pyproject_toml, platform=cast("Any", platform), env={})
    assert options_reader.get("repair-wheel-command") == f"repair-project-{platform}"


def test_unexpected_key(tmp_path: Path) -> None:
    # Note that platform contents are only checked when running
    # for that platform.
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel]
repairs-wheel-command = "repair-project-linux"
"""
    )

    with pytest.raises(OptionsReaderError) as excinfo:
        OptionsReader(pyproject_toml, platform="linux", env={})

    assert "repair-wheel-command" in str(excinfo.value)


def test_underscores_in_key(tmp_path: Path) -> None:
    # Note that platform contents are only checked when running
    # for that platform.
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel]
repair_wheel_command = "repair-project-linux"
"""
    )

    with pytest.raises(OptionsReaderError) as excinfo:
        OptionsReader(pyproject_toml, platform="linux", env={})

    assert "repair-wheel-command" in str(excinfo.value)


def test_unexpected_table(tmp_path: Path) -> None:
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel.linus]
repair-wheel-command = "repair-project-linux"
"""
    )
    with pytest.raises(OptionsReaderError):
        OptionsReader(pyproject_toml, platform="linux", env={})


def test_unsupported_join(tmp_path: Path) -> None:
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel]
build = ["1", "2"]
"""
    )
    options_reader = OptionsReader(pyproject_toml, platform="linux", env={})

    assert options_reader.get("build", option_format=ListFormat(", ")) == "1, 2"
    with pytest.raises(OptionsReaderError):
        options_reader.get("build")


def test_disallowed_a(tmp_path: Path) -> None:
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel.windows]
manylinux-x86_64-image = "manylinux1"
"""
    )
    disallow = {"windows": {"manylinux-x86_64-image"}}
    OptionsReader(pyproject_toml, platform="linux", disallow=disallow, env={})
    with pytest.raises(OptionsReaderError):
        OptionsReader(pyproject_toml, platform="windows", disallow=disallow, env={})


def test_environment_override_empty(tmp_path: Path) -> None:
    pyproject_toml = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """
[tool.cibuildwheel]
manylinux-i686-image = "manylinux1"
manylinux-x86_64-image = ""
"""
    )

    options_reader = OptionsReader(
        pyproject_toml,
        platform="linux",
        env={
            "CIBW_MANYLINUX_I686_IMAGE": "",
            "CIBW_MANYLINUX_AARCH64_IMAGE": "manylinux1",
            "CIBW_XBUILD_TOOLS": "",
        },
    )

    assert options_reader.get("manylinux-x86_64-image") == ""
    assert options_reader.get("manylinux-i686-image") == ""
    assert options_reader.get("manylinux-aarch64-image") == "manylinux1"

    assert options_reader.get("manylinux-x86_64-image", ignore_empty=True) == "manylinux_2_28"
    assert options_reader.get("manylinux-i686-image", ignore_empty=True) == "manylinux1"
    assert options_reader.get("manylinux-aarch64-image", ignore_empty=True) == "manylinux1"

    assert (
        options_reader.get("xbuild-tools", option_format=ListFormat(" ", quote=shlex.quote)) == ""
    )


@pytest.mark.parametrize("ignore_empty", [True, False], ids=["ignore_empty", "no_ignore_empty"])
def test_resolve_cascade(ignore_empty: bool) -> None:
    answer = _resolve_cascade(
        ("not", InheritRule.NONE),
        (None, InheritRule.NONE),
        ("", InheritRule.NONE),
        (None, InheritRule.NONE),
        ignore_empty=ignore_empty,
    )
    assert answer == ("not" if ignore_empty else "")

    answer = _resolve_cascade(
        ("d4", InheritRule.NONE),
        (None, InheritRule.NONE),
        ("that", InheritRule.NONE),
        (None, InheritRule.NONE),
        ignore_empty=ignore_empty,
    )
    assert answer == "that"

    with pytest.raises(ValueError, match="a setting should at least have a default value"):
        _resolve_cascade(
            (None, InheritRule.NONE),
            (None, InheritRule.NONE),
            (None, InheritRule.NONE),
            (None, InheritRule.NONE),
            ignore_empty=ignore_empty,
        )


@pytest.mark.parametrize("ignore_empty", [True, False], ids=["ignore_empty", "no_ignore_empty"])
@pytest.mark.parametrize("rule", [InheritRule.PREPEND, InheritRule.NONE, InheritRule.APPEND])
def test_resolve_cascade_merge_list(ignore_empty: bool, rule: InheritRule) -> None:
    answer = _resolve_cascade(
        (["a1", "a2"], InheritRule.NONE),
        ([], InheritRule.NONE),
        (["b1", "b2"], rule),
        (None, InheritRule.NONE),
        ignore_empty=ignore_empty,
        option_format=ListFormat(" "),
    )

    if not ignore_empty:
        assert answer == "b1 b2"
    elif rule == InheritRule.PREPEND:
        assert answer == "b1 b2 a1 a2"
    elif rule == InheritRule.NONE:
        assert answer == "b1 b2"
    elif rule == InheritRule.APPEND:
        assert answer == "a1 a2 b1 b2"


@pytest.mark.parametrize("rule", [InheritRule.PREPEND, InheritRule.NONE, InheritRule.APPEND])
def test_resolve_cascade_merge_dict(rule: InheritRule) -> None:
    answer = _resolve_cascade(
        ({"value": "a1", "base": "b1"}, InheritRule.NONE),
        (None, InheritRule.NONE),
        ({"value": "override"}, rule),
        (None, InheritRule.NONE),
        option_format=ShlexTableFormat(),
    )

    if rule == InheritRule.PREPEND:
        assert answer == "value=a1 base=b1"
    elif rule == InheritRule.NONE:
        assert answer == "value=override"
    elif rule == InheritRule.APPEND:
        assert answer == "value=override base=b1"


def test_resolve_cascade_merge_strings() -> None:
    answer = _resolve_cascade(
        ("value=a1 base=b1", InheritRule.NONE),
        ("value=override", InheritRule.APPEND),
        option_format=ShlexTableFormat(),
    )
    assert answer == "value=override base=b1"


def test_resolve_cascade_merge_different_types() -> None:
    answer = _resolve_cascade(
        ("value=a1 base=b1", InheritRule.NONE),
        ({"value": "override"}, InheritRule.APPEND),
        ("extra_string_var=c1", InheritRule.APPEND),
        option_format=ShlexTableFormat(),
    )
    assert answer == "value=override base=b1 extra_string_var=c1"


PYPROJECT_2 = """
[tool.cibuildwheel]
build = ["cp38*", "cp37*"]
environment = {FOO="BAR", "HAM"="EGGS"}

test-command = ["pyproject"]

manylinux-x86_64-image = "manylinux1"

[tool.cibuildwheel.macos]
test-requires = "else"

[[tool.cibuildwheel.overrides]]
select = "cp37*"
inherit = {test-command="prepend", environment="append"}
test-command = ["pyproject-override", "override2"]
manylinux-x86_64-image = "manylinux2014"
environment = {FOO="BAZ", "PYTHON"="MONTY"}

[[tool.cibuildwheel.overrides]]
select = "*-final"
inherit = {test-command="append"}
test-command = ["pyproject-finalize", "finalize2"]

[[tool.cibuildwheel.overrides]]
select = "*-final"
inherit = {test-command="append"}
test-command = ["extra-finalize"]

[[tool.cibuildwheel.overrides]]
select = "*-final"
inherit = {test-command="prepend"}
test-command = ["extra-prepend"]
"""


def test_pyproject_2(tmp_path: Path, platform: str) -> None:
    pyproject_toml: Path = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(PYPROJECT_2)

    options_reader = OptionsReader(
        config_file_path=pyproject_toml, platform=cast("Any", platform), env={}
    )
    assert options_reader.get("test-command", option_format=ListFormat(" && ")) == "pyproject"

    with options_reader.identifier("random"):
        assert options_reader.get("test-command", option_format=ListFormat(" && ")) == "pyproject"

    with options_reader.identifier("cp37-something"):
        assert (
            options_reader.get("test-command", option_format=ListFormat(" && "))
            == "pyproject-override && override2 && pyproject"
        )
        assert (
            options_reader.get("environment", option_format=EnvironmentFormat())
            == 'FOO="BAR" HAM="EGGS" FOO="BAZ" PYTHON="MONTY"'
        )

    with options_reader.identifier("cp37-final"):
        assert (
            options_reader.get("test-command", option_format=ListFormat(" && "))
            == "extra-prepend && pyproject-override && override2 && pyproject && pyproject-finalize && finalize2 && extra-finalize"
        )
        assert (
            options_reader.get("environment", option_format=EnvironmentFormat())
            == 'FOO="BAR" HAM="EGGS" FOO="BAZ" PYTHON="MONTY"'
        )


def test_overrides_not_a_list(tmp_path: Path, platform: str) -> None:
    pyproject_toml: Path = tmp_path / "pyproject.toml"

    pyproject_toml.write_text(
        """\
[tool.cibuildwheel]
build = ["cp38*", "cp37*"]
[tool.cibuildwheel.overrides]
select = "cp37*"
test-command = "pyproject-override"
"""
    )

    with pytest.raises(OptionsReaderError):
        OptionsReader(config_file_path=pyproject_toml, platform=cast("Any", platform), env={})


def test_config_settings(tmp_path: Path) -> None:
    pyproject_toml: Path = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """\
[tool.cibuildwheel.config-settings]
example = "one"
other = ["two", "three"]
"""
    )

    options_reader = OptionsReader(config_file_path=pyproject_toml, platform="linux", env={})
    assert (
        options_reader.get("config-settings", option_format=ShlexTableFormat(pair_sep="=", sep=" "))
        == "example=one other=two other=three"
    )


def test_pip_config_settings(tmp_path: Path) -> None:
    pyproject_toml: Path = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """\
[tool.cibuildwheel.config-settings]
--build-option="--use-mypyc"
"""
    )

    options_reader = OptionsReader(config_file_path=pyproject_toml, platform="linux", env={})
    assert (
        options_reader.get("config-settings", option_format=ShlexTableFormat(sep=" ", pair_sep="="))
        == "--build-option=--use-mypyc"
    )


def test_overrides_inherit(tmp_path: Path) -> None:
    pyproject_toml: Path = tmp_path / "pyproject.toml"
    pyproject_toml.write_text(
        """\
[tool.cibuildwheel]
before-all = ["before-all"]
config-settings = {key1="value1", key2="value2", empty=""}

[[tool.cibuildwheel.overrides]]
select = "cp37*"
inherit.before-all = "append"
before-all = ["override1"]

inherit.config-settings = "append"
config-settings = {key3="value3", key2="override2"}

[[tool.cibuildwheel.overrides]]
select = "cp37*"
inherit.before-all = "prepend"
before-all = ["override2"]
"""
    )

    options_reader = OptionsReader(config_file_path=pyproject_toml, platform="linux", env={})
    with options_reader.identifier("cp38-something"):
        assert options_reader.get("before-all", option_format=ListFormat(" && ")) == "before-all"
        assert (
            options_reader.get("config-settings", option_format=ShlexTableFormat())
            == "key1=value1 key2=value2 empty=''"
        )
    with options_reader.identifier("cp37-something"):
        assert (
            options_reader.get("before-all", option_format=ListFormat(" && "))
            == "override2 && before-all && override1"
        )
        assert (
            options_reader.get("config-settings", option_format=ShlexTableFormat())
            == "key1=value1 key2=override2 empty='' key3=value3"
        )



import os
import re
import subprocess
import sysconfig
from typing import Any

import rich.console
import rich.text


def define_env(env: Any) -> None:  # noqa: ANN401
    "Hook function for mkdocs-macros"

    @env.macro  # type: ignore[untyped-decorator]
    def subprocess_run(*args: str) -> str:
        "Run a subprocess and return the stdout"
        env = os.environ.copy()
        scripts = sysconfig.get_path("scripts")
        env.pop("NO_COLOR", None)
        env["PATH"] = f"{scripts}{os.pathsep}{env.get('PATH', '')}"
        env["PYTHON_COLORS"] = "1"
        output = subprocess.run(args, check=True, capture_output=True, text=True, env=env).stdout
        rich_text = rich.text.Text.from_ansi(output)
        console = rich.console.Console(record=True, force_terminal=True)
        console.print(rich_text)
        page = console.export_html(inline_styles=True)
        result = re.search(r"<body.*?>(.*?)</body>", page, re.DOTALL | re.IGNORECASE)
        assert result
        txt = result.group(1).strip()
        return txt.replace("code ", 'code class="nohighlight" ')



# Based on: https://github.com/python/cpython/blob/master/Mac/BuildScript/resources/install_certificates.command

# install_certifi.py
#
# sample script to install or update a set of default Root Certificates
# for the ssl module.  Uses the certificates provided by the certifi package:
#       https://pypi.org/project/certifi/

import contextlib
import os
import os.path
import ssl
import stat
import subprocess
import sys

STAT_0o775 = (
    stat.S_IRUSR
    | stat.S_IWUSR
    | stat.S_IXUSR
    | stat.S_IRGRP
    | stat.S_IWGRP
    | stat.S_IXGRP
    | stat.S_IROTH
    | stat.S_IXOTH
)


def main() -> None:
    openssl_dir, openssl_cafile = os.path.split(ssl.get_default_verify_paths().openssl_cafile)
    print(" -- pip install --upgrade certifi")
    subprocess.check_call(
        [sys.executable, "-E", "-s", "-m", "pip", "install", "--upgrade", "certifi"]
    )

    import certifi  # noqa: PLC0415

    # change working directory to the default SSL directory
    os.chdir(openssl_dir)
    relpath_to_certifi_cafile = os.path.relpath(certifi.where())

    print(" -- removing any existing file or link")
    with contextlib.suppress(FileNotFoundError):
        os.remove(openssl_cafile)
    print(" -- creating symlink to certifi certificate bundle")
    os.symlink(relpath_to_certifi_cafile, openssl_cafile)

    print(" -- setting permissions")
    os.chmod(openssl_cafile, STAT_0o775)
    print(" -- update complete")


if __name__ == "__main__":
    main()



import tomllib

from packaging.version import Version

from cibuildwheel.extra import Printable, dump_python_configurations
from cibuildwheel.util import resources


def test_compare_configs() -> None:
    txt = resources.BUILD_PLATFORMS.read_text()

    with resources.BUILD_PLATFORMS.open("rb") as f2:
        dict_txt = tomllib.load(f2)

    new_txt = dump_python_configurations(dict_txt)
    print(new_txt)

    assert new_txt == txt


def test_dump_with_Version() -> None:
    # MyPy doesn't understand deeply nested dicts correctly
    example: dict[str, dict[str, list[dict[str, Printable]]]] = {
        "windows": {
            "python_configurations": [
                {"identifier": "cp27-win32", "version": Version("2.7.18"), "arch": "32"},
                {"identifier": "cp27-win_amd64", "version": "2.7.18", "arch": "64"},
            ]
        }
    }

    result = """\
[windows]
python_configurations = [
  { identifier = "cp27-win32", version = "2.7.18", arch = "32" },
  { identifier = "cp27-win_amd64", version = "2.7.18", arch = "64" },
]
"""

    output = dump_python_configurations(example)
    print(output)
    assert output == result



import sys
from pathlib import Path

import pytest

MOCK_PACKAGE_DIR = Path("some_package_dir")


def pytest_addoption(parser: pytest.Parser) -> None:
    parser.addoption("--run-docker", action="store_true", default=False, help="run docker tests")
    parser.addoption("--run-podman", action="store_true", default=False, help="run podman tests")
    parser.addoption(
        "--run-cp38-universal2",
        action="store_true",
        default=False,
        help="macOS cp38 uses the universal2 installer",
    )


@pytest.fixture
def fake_package_dir(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> list[str]:
    """
    Monkey-patch enough for the main() function to run
    """
    real_path_exists = Path.exists

    def mock_path_exists(path: Path) -> bool:
        if str(path).endswith(str(MOCK_PACKAGE_DIR / "setup.py")):
            return True
        else:
            return real_path_exists(path)

    args = ["cibuildwheel", str(MOCK_PACKAGE_DIR)]
    tmp_path.joinpath(MOCK_PACKAGE_DIR).mkdir()
    monkeypatch.setattr(Path, "exists", mock_path_exists)
    monkeypatch.setattr(sys, "argv", args)
    monkeypatch.chdir(tmp_path)
    return args



import sys
from collections.abc import Sequence
from pathlib import Path
from typing import Final, Protocol

from cibuildwheel import errors
from cibuildwheel.architecture import Architecture
from cibuildwheel.options import Options
from cibuildwheel.platforms import android, ios, linux, macos, pyodide, windows
from cibuildwheel.selector import BuildSelector
from cibuildwheel.typing import GenericPythonConfiguration, PlatformName


class PlatformModule(Protocol):
    # note that as per PEP544, the self argument is ignored when the protocol
    # is applied to a module
    def all_python_configurations(self) -> Sequence[GenericPythonConfiguration]: ...

    def get_python_configurations(
        self, build_selector: BuildSelector, architectures: set[Architecture]
    ) -> Sequence[GenericPythonConfiguration]: ...

    def build(self, options: Options, tmp_path: Path) -> None: ...


ALL_PLATFORM_MODULES: Final[dict[PlatformName, PlatformModule]] = {
    "linux": linux,
    "windows": windows,
    "macos": macos,
    "pyodide": pyodide,
    "android": android,
    "ios": ios,
}


def native_platform() -> PlatformName:
    if sys.platform.startswith("linux"):
        return "linux"
    elif sys.platform == "darwin":
        return "macos"
    elif sys.platform == "win32":
        return "windows"
    else:
        msg = (
            'Unable to detect platform from "sys.platform". cibuildwheel doesn\'t '
            "support building wheels for this platform. You might be able to build for a different "
            "platform using the --platform argument. Check --help output for more information."
        )
        raise errors.ConfigurationError(msg)


def get_build_identifiers(
    platform_module: PlatformModule,
    build_selector: BuildSelector,
    architectures: set[Architecture],
) -> list[str]:
    python_configurations = platform_module.get_python_configurations(build_selector, architectures)
    return [config.identifier for config in python_configurations]



#!/usr/bin/env python3


import dataclasses
import difflib
import logging
import tomllib
from pathlib import Path
from typing import Final

import click
import rich
from packaging.version import Version
from rich.logging import RichHandler
from rich.syntax import Syntax

from cibuildwheel.extra import github_api_request

log = logging.getLogger("cibw")

# Looking up the dir instead of using utils.resources_dir
# since we want to write to it.
DIR: Final[Path] = Path(__file__).parent.parent.resolve()
RESOURCES_DIR: Final[Path] = DIR / "cibuildwheel/resources"

GET_VIRTUALENV_GITHUB: Final[str] = "https://github.com/pypa/get-virtualenv"
GET_VIRTUALENV_URL_TEMPLATE: Final[str] = (
    f"{GET_VIRTUALENV_GITHUB}/blob/{{version}}/public/virtualenv.pyz?raw=true"
)


@dataclasses.dataclass(frozen=True, order=True)
class VersionTuple:
    name: str
    download_url: str
    version: Version


def get_latest_virtualenv_release() -> VersionTuple:
    response = github_api_request("repos/pypa/get-virtualenv/releases/latest")
    tag_name = response["tag_name"]

    asset = next(
        (asset for asset in response["assets"] if asset["name"] == "virtualenv.pyz"),
        None,
    )
    if not asset:
        msg = "No asset named 'virtualenv.pyz' found in the latest release of get-virtualenv."
        raise RuntimeError(msg)

    return VersionTuple(
        version=Version(tag_name), name=tag_name, download_url=asset["browser_download_url"]
    )


@click.command()
@click.option("--force", is_flag=True)
@click.option(
    "--level", default="INFO", type=click.Choice(["WARNING", "INFO", "DEBUG"], case_sensitive=False)
)
def update_virtualenv(force: bool, level: str) -> None:
    logging.basicConfig(
        level="INFO",
        format="%(message)s",
        datefmt="[%X]",
        handlers=[RichHandler(rich_tracebacks=True, markup=True)],
    )
    log.setLevel(level)

    toml_file_path = RESOURCES_DIR / "virtualenv.toml"

    original_toml = toml_file_path.read_text()
    with toml_file_path.open("rb") as f:
        configurations = tomllib.load(f)
    default = configurations.pop("default")
    local_version = str(default["version"])

    latest_release = get_latest_virtualenv_release()

    if latest_release.version > Version(local_version):
        version = latest_release.name
        url = latest_release.download_url
    else:
        version = local_version
        url = default["url"]

    configurations["default"] = {
        "version": version,
        "url": url,
    }
    result_toml = "".join(
        f'{key} = {{ version = "{value["version"]}", url = "{value["url"]}" }}\n'
        for key, value in configurations.items()
    )

    rich.print()  # spacer

    if original_toml == result_toml:
        rich.print("[green]Check complete, virtualenv version unchanged.")
        return

    rich.print("virtualenv version updated.")
    rich.print("Changes:")
    rich.print()

    toml_relpath = toml_file_path.relative_to(DIR).as_posix()
    diff_lines = difflib.unified_diff(
        original_toml.splitlines(keepends=True),
        result_toml.splitlines(keepends=True),
        fromfile=toml_relpath,
        tofile=toml_relpath,
    )
    rich.print(Syntax("".join(diff_lines), "diff", theme="ansi_light"))
    rich.print()

    if force:
        toml_file_path.write_text(result_toml)
        rich.print("[green]TOML file updated.")
    else:
        rich.print("[yellow]File left unchanged. Use --force flag to update.")


if __name__ == "__main__":
    update_virtualenv()



#!/usr/bin/env python3


import argparse
import os
import subprocess
import sys
from pathlib import Path

if __name__ == "__main__":
    if sys.version_info < (3, 13):
        default_cpu_count = os.cpu_count() or 2
    else:
        default_cpu_count = os.process_cpu_count() or 2

    parser = argparse.ArgumentParser(allow_abbrev=False)
    parser.add_argument(
        "--run-podman", action="store_true", default=False, help="run podman tests (linux only)"
    )
    parser.add_argument(
        "--num-processes",
        action="store",
        default=default_cpu_count,
        help="number of processes to use for testing",
    )
    parser.add_argument(
        "--test-select",
        choices={"all", "native", "android", "ios", "pyodide"},
        default="all",
        help="Either 'native' or 'android'/'ios'/'pyodide'",
    )
    args = parser.parse_args()

    # move cwd to the project root
    os.chdir(Path(__file__).resolve().parents[1])

    # doc tests
    doc_test_args = [sys.executable, "-m", "pytest", "cibuildwheel"]

    print(
        "\n\n================================== DOC TESTS ==================================",
        flush=True,
    )
    result = subprocess.run(doc_test_args, check=False)
    if result.returncode not in (0, 5):
        # Allow case where no doctests are collected (returncode 5) because
        # circleci sets an explicit "-k" filter that disables doctests. There
        # isn't a pattern that will only select doctests. This can be removed
        # and have check=True if the circleci PYTEST_ADDOPTS is removed.
        raise subprocess.CalledProcessError(
            result.returncode, result.args, output=result.stdout, stderr=result.stderr
        )

    # unit tests
    print(
        "\n\n================================== UNIT TESTS ==================================",
        flush=True,
    )
    unit_test_args = [sys.executable, "-m", "pytest", "unit_test"]

    if (
        sys.platform.startswith("linux")
        and os.environ.get("CIBW_PLATFORM", "linux") == "linux"
        and args.test_select in ["all", "native"]
    ):
        # run the docker unit tests only on Linux
        unit_test_args += ["--run-docker"]

        if args.run_podman:
            unit_test_args += ["--run-podman"]

    subprocess.run(unit_test_args, check=True)

    print(
        "\n\n=========================== SERIAL INTEGRATION TESTS ===========================",
        flush=True,
    )

    match args.test_select:
        case "all":
            marks = []
        case "native":
            marks = ["not pyodide", "not android", "not ios"]
        case mark:
            marks = [f"{mark}"]

    # Run the serial integration tests without multiple processes
    serial_integration_test_args = [
        sys.executable,
        "-m",
        "pytest",
        "-m",
        f"{' and '.join(['serial', *marks])}",
        "-x",
        "--durations",
        "0",
        "--timeout=2400",
        "test",
        "-vv",
    ]

    subprocess.run(serial_integration_test_args, check=True)

    print(
        "\n\n========================= NON-SERIAL INTEGRATION TESTS =========================",
        flush=True,
    )
    integration_test_args = [
        sys.executable,
        "-m",
        "pytest",
        "-m",
        f"{' and '.join(['not serial', *marks])}",
        f"--numprocesses={args.num_processes}",
        "-x",
        "--durations",
        "0",
        "--timeout=2400",
        "test",
        "-vv",
    ]

    if sys.platform.startswith("linux") and args.run_podman:
        integration_test_args += ["--run-podman"]

    subprocess.run(integration_test_args, check=True)



import dataclasses
import subprocess
from collections.abc import (
    Callable,
    Iterable,
    Mapping,
    Sequence,
)

import bashlex

# a function that takes a command and the environment, and returns the result
EnvironmentExecutor = Callable[[list[str], dict[str, str]], str]


def local_environment_executor(command: Sequence[str], env: Mapping[str, str]) -> str:
    return subprocess.run(command, env=env, text=True, stdout=subprocess.PIPE, check=True).stdout


@dataclasses.dataclass(frozen=True, kw_only=True)
class NodeExecutionContext:
    environment: dict[str, str]
    input: str
    executor: EnvironmentExecutor


def evaluate(
    value: str, environment: Mapping[str, str], executor: EnvironmentExecutor | None = None
) -> str:
    if not value:
        # empty string evaluates to empty string
        # (but trips up bashlex)
        return ""

    command_node = bashlex.parsesingle(value)

    if len(command_node.parts) != 1:
        msg = f"{value!r} has too many parts"
        raise ValueError(msg)

    value_word_node = command_node.parts[0]

    return evaluate_node(
        value_word_node,
        context=NodeExecutionContext(
            environment=dict(environment),
            input=value,
            executor=executor or local_environment_executor,
        ),
    )


def evaluate_node(node: bashlex.ast.node, context: NodeExecutionContext) -> str:
    if node.kind == "word":
        return evaluate_word_node(node, context=context)
    elif node.kind == "commandsubstitution":
        node_result = evaluate_command_node(node.command, context=context)
        # bash removes training newlines in command substitution
        return node_result.rstrip()
    elif node.kind == "parameter":
        return evaluate_parameter_node(node, context=context)
    else:
        msg = f"Unsupported bash construct: {node.kind!r}"
        raise ValueError(msg)


def evaluate_word_node(node: bashlex.ast.node, context: NodeExecutionContext) -> str:
    value: str = node.word

    for part in node.parts:
        part_string = context.input[part.pos[0] : part.pos[1]]
        part_value = evaluate_node(part, context=context)

        if part_string not in value:
            msg = f"bash parse failed. part {part_string!r} not found in {value!r}. Word was {node.word!r}. Full input was {context.input!r}"
            raise RuntimeError(msg)

        value = value.replace(part_string, part_value, 1)

    return value


def evaluate_command_node(node: bashlex.ast.node, context: NodeExecutionContext) -> str:
    if any(n.kind == "operator" for n in node.parts):
        return evaluate_nodes_as_compound_command(node.parts, context=context)
    else:
        return evaluate_nodes_as_simple_command(node.parts, context=context)


def evaluate_nodes_as_compound_command(
    nodes: Sequence[bashlex.ast.node], context: NodeExecutionContext
) -> str:
    # bashlex doesn't support any operators besides ';' inside command
    # substitutions, so we only need to handle that case. We do so assuming
    # that `set -o errexit` is on, because it's easier to code!

    result = ""
    for node in nodes:
        if node.kind == "command":
            result += evaluate_command_node(node, context=context)
        elif node.kind == "operator":
            if node.op != ";":
                msg = f"Unsupported bash operator: {node.op!r}"
                raise ValueError(msg)
        else:
            msg = f"Unsupported bash node in compound command: {node.kind!r}"
            raise ValueError(msg)

    return result


def evaluate_nodes_as_simple_command(
    nodes: Iterable[bashlex.ast.node], context: NodeExecutionContext
) -> str:
    command = [evaluate_node(part, context=context) for part in nodes]
    return context.executor(command, context.environment)


def evaluate_parameter_node(node: bashlex.ast.node, context: NodeExecutionContext) -> str:
    return context.environment.get(node.value, "")



"""
Errors that can cause the build to fail. Each subclass of FatalError has
a different return code, by defining them all here, we can ensure that they're
semantically clear and unique.
"""

import textwrap


class FatalError(BaseException):
    """
    Raising an error of this type will cause the message to be printed to
    stderr and the process to be terminated. Within cibuildwheel, raising this
    exception produces a better error message, and optional traceback.
    """

    return_code: int = 1


class ConfigurationError(FatalError):
    return_code = 2


class NothingToDoError(FatalError):
    return_code = 3


class DeprecationError(FatalError):
    return_code = 4


class NonPlatformWheelError(FatalError):
    def __init__(self) -> None:
        message = textwrap.dedent(
            """
            Build failed because a pure Python wheel was generated.

            If you intend to build a pure-Python wheel, you don't need
            cibuildwheel - use `pip wheel .`, `pipx run build --wheel`, `uv
            build --wheel`, etc. instead. You only need cibuildwheel if you
            have compiled (not Python) code in your wheels making them depend
            on the platform.

            If you expected a platform wheel, check your project configuration,
            or run cibuildwheel with CIBW_BUILD_VERBOSITY=1 to view build logs.
            """
        )
        super().__init__(message)
        self.return_code = 5


class AlreadyBuiltWheelError(FatalError):
    def __init__(self, wheel_name: str) -> None:
        message = textwrap.dedent(
            f"""
            Build failed because a wheel named {wheel_name} was already generated in the current run.

            If you expected another wheel to be generated, check your project configuration, or run
            cibuildwheel with CIBW_BUILD_VERBOSITY=1 to view build logs.
            """
        )
        super().__init__(message)
        self.return_code = 6


class OCIEngineTooOldError(FatalError):
    def __init__(self, message: str) -> None:
        super().__init__(message)
        self.return_code = 7


class RepairStepProducedNoWheelError(FatalError):
    def __init__(self) -> None:
        message = textwrap.dedent(
            """
            Build failed because the repair step completed successfully but
            did not produce a wheel.

            Your `repair-wheel-command` is expected to place the repaired
            wheel in the {dest_dir} directory. See the documentation for
            example configurations:

            https://cibuildwheel.pypa.io/en/stable/options/#repair-wheel-command
            """
        )
        super().__init__(message)
        self.return_code = 8


class RepairStepProducedMultipleWheelsError(FatalError):
    def __init__(self, wheels: list[str]) -> None:
        message = textwrap.dedent(
            f"""
            Build failed because the repair step completed successfully but
            produced multiple wheels: {wheels}

            Your `repair-wheel-command` is expected to place one repaired
            wheel in the {{dest_dir}} directory. See the documentation for
            example configurations:

            https://cibuildwheel.pypa.io/en/stable/options/#repair-wheel-command
            """
        )
        super().__init__(message)
        self.return_code = 8



import os
import subprocess
import textwrap
from pathlib import Path

import pytest

from . import test_projects, utils

basic_project = test_projects.new_c_project()


def skip_if_no_msvc(arm64: bool = False) -> None:
    programfiles = os.getenv("PROGRAMFILES(X86)", "") or os.getenv("PROGRAMFILES", "")
    if not programfiles:
        pytest.skip("Requires %PROGRAMFILES(X86)% variable to be set")

    vswhere = Path(programfiles, "Microsoft Visual Studio", "Installer", "vswhere.exe")
    if not vswhere.is_file():
        pytest.skip("Requires Visual Studio installation")

    require = "Microsoft.VisualStudio.Component.VC.Tools.x86.x64"
    if arm64:
        require = "Microsoft.VisualStudio.Component.VC.Tools.ARM64"

    if not subprocess.check_output(
        [
            vswhere,
            "-latest",
            "-prerelease",
            "-property",
            "installationPath",
            "-requires",
            require,
        ]
    ):
        pytest.skip("Requires ARM64 compiler to be installed")


@pytest.mark.parametrize("use_pyproject_toml", [True, False])
def test_wheel_tag_is_correct_when_using_windows_cross_compile(
    tmp_path: Path, use_pyproject_toml: bool
) -> None:
    if utils.get_platform() != "windows":
        pytest.skip("This test is only relevant to Windows")

    skip_if_no_msvc(arm64=True)

    if use_pyproject_toml:
        basic_project.files["pyproject.toml"] = textwrap.dedent(
            """
            [build-system]
            requires = ["setuptools"]
            build-backend = "setuptools.build_meta"
            """
        )

    project_dir = tmp_path / "project"
    basic_project.generate(project_dir)

    # build the wheels
    actual_wheels = utils.cibuildwheel_run(
        project_dir,
        add_args=["--archs", "ARM64"],
        single_python=True,
    )

    # check that the expected wheels are produced
    tag = "cp{}{}".format(*utils.SINGLE_PYTHON_VERSION)
    expected_wheels = [f"spam-0.1.0-{tag}-{tag}-win_arm64.whl"]

    print("actual_wheels", actual_wheels)
    print("expected_wheels", expected_wheels)

    assert set(actual_wheels) == set(expected_wheels)



import subprocess
from pathlib import Path

import pytest

from test import test_projects

from . import utils

pure_python_project = test_projects.TestProject()
pure_python_project.files["setup.py"] = """
from setuptools import Extension, setup

setup(
    name="spam",
    py_modules=['spam'],
    version="0.1.0",
)
"""

pure_python_project.files["spam.py"] = """
def a_function():
    pass
"""


def test(tmp_path: Path, capfd: pytest.CaptureFixture[str]) -> None:
    # this test checks that if a pure wheel is generated, the build should
    # fail.
    project_dir = tmp_path / "project"
    pure_python_project.generate(project_dir)

    with pytest.raises(subprocess.CalledProcessError) as exc_info:
        print("produced wheels:", utils.cibuildwheel_run(project_dir))

    captured = capfd.readouterr()
    print("out", captured.out)
    print("err", captured.err)
    assert exc_info.value.returncode == 5
    assert "Build failed because a pure Python wheel was generated" in captured.err



#!/usr/bin/env python3

import re
from pathlib import Path

PROJECT_ROOT = Path(__file__).parent / ".."
CHANGELOG_FILE = PROJECT_ROOT / "docs" / "changelog.md"

# https://regexr.com/622ds
FIRST_5_CHANGELOG_ENTRIES_REGEX = re.compile(r"""(^###.*?(?=###)){5}""", re.DOTALL | re.MULTILINE)


def mini_changelog() -> str:
    changelog_text = CHANGELOG_FILE.read_text()

    mini_changelog_match = FIRST_5_CHANGELOG_ENTRIES_REGEX.search(changelog_text)
    assert mini_changelog_match, "Failed to find the first few changelog entries"

    return f"\n{mini_changelog_match.group(0).strip()}\n"


if __name__ == "__main__":
    print(mini_changelog())
